/****************************************************************************
*                              NOTICE
*  This source code contains proprietary and confidential information,
*  and is the property of the Dialogic Corporation.
*
*  This program has been distributed under license agreement and any
*  unauthorized use or distribution is prohibited.
*
*  Copyright 1999 by Dialogic Corporation.  All rights reserved.
 ***************************************************************************/
/****************************************************************************
 * FILE:         transmanage.h
 * AUTHOR:       
 * ORG:          Dialogic Corporation
 * ORIG-DATE:    04/01/2009
 * LAST-MOD:
 * PURPOSE:      Video Transcoder Management Interface
 *
 *
 * HISTORY:
 * Date        Who  CR:Description
 * 04/06/2010  JD   IPY90900: Support for Out of Band DCI
 * 10/21/2009  JAF  CR34572: Adding new debug/validation support
 ******************************************************************/

#ifndef _TRANSMANAGE_H
#define _TRANSMANAGE_H

/* ----- dependent includes ----- */
#include "vserr.h"                  /* error code base defintions */
#include "vsport.h"                 /* well-known port number defintions */
#include "vstypes.h"                /* basic type definitions */
#include "vslogmsg.h"               /* log message definitions */
#include "trcdefs.h"                /* common transcoder definitions */

/* ----- protocol version ----- */
#define VTMNG_VERSION   1
#define VTMNG_REVISION  1

/* ----- message operations ----- */
#define VTMNG_OP_GET_LIST       0x01    /* retrieve list of entity IDs */
#define VTMNG_OP_GET_ENTITY     0x02    /* retrieve copy of single entity record (config + status + stats) */
#define VTMNG_OP_SET_CONFIG     0x03    /* change an entity's current configuration */
#define VTMNG_OP_ZERO_STATS     0x04    /* zero all statistics (and clear all stored error indications) */
#define VTMNG_OP_NOTIFY         0x05    /* asynchronous notification */

/* ----- message type IDs ----- */
#define VTMNG_REQ_ID            0x01    /* request */
#define VTMNG_RSP_ID            0x02    /* response */
#define VTMNG_IND_ID            0x03    /* asynchronous indication */

/* ----- message categories ----- */
#define VTMNG_CATEG_VTP         0x01    /* VTP-level control */
#define VTMNG_CATEG_APP         0x02    /* Application-level control */
#define VTMNG_CATEG_MON         0x03    /* Monitored Process control */
#define VTMNG_CATEG_CHN         0x04    /* Channel-level control */
#define VTMNG_CATEG_ST_TOTAL    0x05    /* Total statistics */
#define VTMNG_CATEG_ST_CURRMIN  0x06    /* Current Minute statistics */
#define VTMNG_CATEG_HIST_PERMIN 0x07    /* Histogram statistics [per-minute increments] */
#define VTMNG_CATEG_HIST_PERHHR 0x08    /* Histogram statistics [per-half-hour increments] */

/*****************************************************************************
 * REQUEST MATRIX - table showing valid operation+category combinations
 *
 *      XXXXXX = not allowed
 *      SINGLE = valid request; response contains single record
 *      [LIST] = valid request; response contains list of records
 *
 *   \  CATEG| VTP    | APP or | CHN    | ST     | ST     | HIST   | HIST   |
 * OP  \     |        | MON    |        | TOTAL  | CURRMIN| PERMIN | PERHHR |
 * --------------------------------------------------------------------------
 * GET_LIST  | XXXXXX | [LIST] | [LIST] | XXXXXX | XXXXXX | XXXXXX | XXXXXX |
 * --------------------------------------------------------------------------
 * GET_ENTITY| SINGLE | SINGLE | SINGLE | SINGLE | SINGLE | SINGLE | SINGLE |
 * --------------------------------------------------------------------------
 * SET_CONFIG| SINGLE | SINGLE | SINGLE | XXXXXX | XXXXXX | XXXXXX | XXXXXX |
 * --------------------------------------------------------------------------
 * ZERO_STATS| SINGLE | SINGLE | SINGLE | SINGLE | XXXXXX | XXXXXX | XXXXXX |
 * --------------------------------------------------------------------------
 *
 *****************************************************************************/

/* ----- types of optional licensing ----- */
#define VTMNG_LICENSE_OVERLAY   (1<<0)  /* set = licensed for overlays */

/* ----- field count / size limits ----- */
#define VTMNG_NAME_SZ           64      /* space reserved for ASCII name */
#define VTMNG_DESC_SZ           96      /* space reserved for ASCII desciption */
#define VTMNG_LONGNAME_SZ       128     /* space reserved for long ASCII names */
#define VTMNG_MAX_MESSAGE       102400    /* largest message size */
#define VTMNG_MON_VARS          8       /* number of per-process variables that can be stored by process monitor */

/* ----- reserved values ----- */
#define VTMNG_RESERVED_SENDERID 0xFFFFFFFF  /* special senderID reserved for internal use */
#define VTMNG_TRCAGENT_SENDERID 0xFFFFFFFE  /* special senderID indicating the trc_agent */
#define VTMNG_TRCP_SENDERID     0xFFFFFFFD  /* special senderID indicating the TRCP */
#define VTMNG_NO_CHANGE         0           /* used to indicate no change in given field (mask for set) */
#define VTMNG_CHANGE            1           /* used to indicate that a given field should change (mask for set) */
#define VTMNG_CFG_DISABLED      0           /* configurable feature is disabled */
#define VTMNG_CFG_ENABLED       1           /* configurable feature is enabled */
#define VTMNG_CFG_INTERNAL      0x40        /* set configurable value using internal algorithm [ASCII '@' in text files] */
#define VTMNG_CFG_AUTO_ADDRESS  0x26        /* set address automatically [ASCII '&' character] */

/* ----- trap (asynchronous notification) events issued ----- */
#define VTMNG_EVENT_VTP_LICENSE (1<<0)  /* license usage threshold crossed */
#define VTMNG_EVENT_VTP_USAGE   (1<<1)  /* estimated CPU usage threshold crossed */
#define VTMNG_EVENT_VTP_ERROR   (1<<2)  /* VTP-level error detected */

#define VTMNG_EVENT_APP_CONNECT (1<<8)  /* connection established to controlling application */
#define VTMNG_EVENT_APP_NAMED   (1<<9)  /* controlling application name has been discovered */
#define VTMNG_EVENT_APP_DISCON  (1<<10) /* previously established application connection lost */
#define VTMNG_EVENT_APP_DEAD    (1<<11) /* application connection destroyed */

#define VTMNG_EVENT_CHN_CREATE  (1<<16) /* channel created */
#define VTMNG_EVENT_CHN_START   (1<<17) /* channel started */
#define VTMNG_EVENT_CHN_STOP    (1<<18) /* channel stopped */
#define VTMNG_EVENT_CHN_DEAD    (1<<19) /* channel terminated */
#define VTMNG_EVENT_CHN_ERROR   (1<<20) /* channnel-level error detected */

#define VTMNG_EVENT_MON_LOST    (1<<24) /* monitored process has been lost (recovery in progress) */
#define VTMNG_EVENT_MON_RECOVER (1<<25) /* monitored process has been recovered */

/* ----- VTP top-level states ----- */
#define VTMNG_VTP_S_INITIAL     0x01    /* initializing */
#define VTMNG_VTP_S_ENABLED     0x02    /* enabled (allowing new channel assignments) */
#define VTMNG_VTP_S_DISABLED    0x03    /* disabled (no current channels and no new channels allowed) */
#define VTMNG_VTP_S_DISABLING   0x04    /* waiting for current channels to complete before going disabled */
#define VTMNG_VTP_S_ABORTING    0x05    /* forcing all channels to terminate immediately (hard disable) */

/* ----- VTP top-level state changing events ----- */
#define VTMNG_VTP_E_ENABLE      0x01    /* enable assignment of channels to this VTP */
#define VTMNG_VTP_E_DISABLE     0x02    /* do not allow new channel assignments; once idle go to disabled state */
#define VTMNG_VTP_E_ABORT       0x03    /* terminate all channels; once all aborted go to disabled state */
#define VTMNG_VTP_E_RESTART     0x04    /* stop all transcoder processes and then restart all processes */
#define VTMNG_VTP_E_REBOOT      0x05    /* cause the VTP to reboot */

/* ----- VTP initialization state choices ----- */
#define VTMNG_VTP_I_MIN         0x01    /* minimum VTP initial state */
#define VTMNG_VTP_I_ENABLED     0x01    /* initialize to the enabled state (allowing channel assignements to the given VTP) */
#define VTMNG_VTP_I_DISABLED    0x02    /* initialize to the disabled state (not allowing channel assignments) */
#define VTMNG_VTP_I_MAX         0x02    /* maximum VTP initial state */

/* ----- Application connection states ----- */
#define VTMNG_APP_S_INITIAL     0x01    /* initializing */
#define VTMNG_APP_S_ACCEPTING   0x02    /* accepting the connection to the client application */
#define VTMNG_APP_S_INIT_WAIT   0x03    /* waiting to receive INIT request from client application */
#define VTMNG_APP_S_CONNECT     0x04    /* connected to controlling application */
#define VTMNG_APP_S_DROP        0x05    /* dropping connection to app (due to error) so that conn. can be re-estab'd */
#define VTMNG_APP_S_LOST_KILL   0x06    /* connection with controlling application has been lost - killing TCP FSM */
#define VTMNG_APP_S_LOST        0x07    /* connection with controlling application has been lost */
#define VTMNG_APP_S_SHUTDOWN    0x08    /* application has requested shutdown - terminating all channels */
#define VTMNG_APP_S_LAST_SEND   0x09    /* waiting for last send complete before disconnecting */
#define VTMNG_APP_S_WAIT_DISCON 0x0A    /* waiting for TRC API to disconnect */
#define VTMNG_APP_S_DISCON      0x0B    /* disconnecting */
#define VTMNG_APP_S_KILL_TCP    0x0C    /* killing TCP FSM */
#define VTMNG_APP_S_KILL_CHN    0x0D    /* quietly killing all channels controlled by this APP FSM */
#define VTMNG_APP_S_DEAD        0x0E    /* control machine had unrecoverable error - no longer accepting events */

/* ----- Monitired Process states ----- */
#define VTMNG_MON_S_CREATING    0x01    /* creating monitored process */
#define VTMNG_MON_S_OWNED       0x02    /* process that was successfully created by monitor (considered owned by monitor) */
#define VTMNG_MON_S_MONITORED   0x03    /* process that is being monitored but was not created by monitor (not recoverable by monitor) */
#define VTMNG_MON_S_CHILD       0x04    /* detected as child process of a different monitored process */
#define VTMNG_MON_S_OWN_LOST    0x05    /* owned process has been lost */
#define VTMNG_MON_S_MON_LOST    0x06    /* monitorde process has been lost */
#define VTMNG_MON_S_CHILD_LOST  0x07    /* child process has been lost */
#define VTMNG_MON_S_ORPHAN      0x08    /* child process now considered an orphan */
#define VTMNG_MON_S_STOPPED     0x09    /* process has been stopped (will not be automatically restarted) */

/* ----- Monitored Process state changing events ----- */
#define VTMNG_MON_E_TERMINATE   0x01    /* terminate the process (allowing auto-recovery to recover the process) */
#define VTMNG_MON_E_STOP        0x02    /* cause the process to stop executing and do not automatically restart the process */
#define VTMNG_MON_E_START       0x03    /* start up a process that had been stopped */

/* ----- Monitored Process initialization state choices ----- */
#define VTMNG_MON_I_MIN         0x01    /* minimum Monitored Process initial state */
#define VTMNG_MON_I_CREATE      0x01    /* create process and then monitor the process */
#define VTMNG_MON_I_LOCATE      0x02    /* locate the process (not created by the video transcoder monitor) */
#define VTMNG_MON_I_SKIP        0x03    /* skip the process (not created and not monitored) */
#define VTMNG_MON_I_MAX         0x03    /* maximum Monitored Process initial state */

/* ----- Monitored Process: action to take on detection of process lost ----- */
#define VTMNG_MON_A_MIN         0x01    /* minimum Monitored Process action */
#define VTMNG_MON_A_NONE        0x01    /* take no action on process lost (just report lost process) */
#define VTMNG_MON_A_PROCRUN     0x02    /* re-run the process */
#define VTMNG_MON_A_DISABLE     0x03    /* trigger VTP disable... once disabled, perform a restart */
#define VTMNG_MON_A_RESTART     0x04    /* perform an immediate restart: stop all processes, then start all */
#define VTMNG_MON_A_TRCPCLEAN   0x05    /* stop all vtmon-created processes and clean up (terminate) all trcp's, then start all */
#define VTMNG_MON_A_MAX         0x05    /* maximum Monitored Process action */

/* ----- Channel states ----- */
#define VTMNG_CHN_S_INITIAL             0x01    /* initializing */
#define VTMNG_CHN_S_CREATING            0x02    /* channel has been assigned a TRCP and has create outstanding */
#define VTMNG_CHN_S_CREATE_DESTROY      0x03    /* channel was being created when destroy requested */
#define VTMNG_CHN_S_CREATE_SHUTDOWN     0x04    /* channel was being created when shutdown requested */
#define VTMNG_CHN_S_CREATE_DIE          0x05    /* channel was being created when die queitly occurred */
#define VTMNG_CHN_S_IDLE                0x06    /* channel has been successfully created but is not started */
#define VTMNG_CHN_S_STARTING            0x07    /* starting all channel resources */
#define VTMNG_CHN_S_ACTIVE              0x08    /* channel has been successfully started and is actively transcoding */
#define VTMNG_CHN_S_STOPPING            0x09    /* stopping all transcoder resources associated with channel */
#define VTMNG_CHN_S_ABORTING            0x0A    /* channel is aborting due to request from management entity */
#define VTMNG_CHN_S_DESTROYING          0x0B    /* DESTROY request outstanding to TRCP */
#define VTMNG_CHN_S_SHUTDOWN            0x0C    /* stopping all resources associated with channel due to SHUTDOWN */
#define VTMNG_CHN_S_DYING               0x0D    /* dying quietly */
#define VTMNG_CHN_S_DEAD                0x0E    /* channel machine has been deallocated */

/* ----- Channel state changing events ----- */
#define VTMNG_CHN_E_ABORT       0x01    /* cause the channel to terminate immediately */

/* ----- Channel Overlay Requirements (options when channel created) ----- */
#define VTMNG_CHN_OVL_ANY       0x00    /* channel does not use overlays; accept assignment to any VTP (except overlay exclusive) */
#define VTMNG_CHN_OVL_REQUIRED  0x01    /* channel requires overlays; restrict to VTPs licensed for overlays */
#define VTMNG_CHN_OVL_NEVER     0x02    /* channel does not use overlays; restrict to VTPs not licensed for overlays */

/* ----- Histogram bucket counts ----- */
#define VTMNG_ALL_BUCKETS       0       /* special indicator used to retrieve all entries in single response message */
#define VTMNG_MAX_MIN_BUCKET    60      /* maximum history depth for per-minute histogram records */
#define VTMNG_MAX_HHR_BUCKET    48      /* maximum history depth for per-half-hour historgram records */

/* ----- Management Error codes ----- */
#define VTMNG_ERRCODE(unique) VS_ERROR_CODE(VS_SYSTEM_VIDEO_XCODER,VX_MNG_BASE,unique)

#define VTMNG_ERR_UNKNOWN_TYPE      (VTMNG_ERRCODE(VS_REASON_INVALID_PARAM))    /* unknown field type encountered */
#define VTMNG_ERR_NOT_INITIALIZED   (VTMNG_ERRCODE(VS_REASON_NOT_INITIALIZED))  /* module has not been initialized */
#define VTMNG_ERR_INVALID_STATE     (VTMNG_ERRCODE(VS_REASON_INVALID_STATE))    /* invalid state for requested operation */
#define VTMNG_ERR_OUT_OF_MEMORY     (VTMNG_ERRCODE(VS_REASON_OUT_OF_MEMORY))    /* insufficient memory to complete request */
#define VTMNG_ERR_OUT_OF_RANGE      (VTMNG_ERRCODE(VS_REASON_OUT_OF_RANGE))     /* value encountered that is out of valid range */
#define VTMNG_ERR_NO_RESOURCE       (VTMNG_ERRCODE(VS_REASON_NO_RESOURCE))      /* unable to obtain required resource */
#define VTMNG_ERR_NOT_IMPLEMENTED   (VTMNG_ERRCODE(VS_REASON_NOT_IMPLEMENTED))  /* requested function not yet implemented */
#define VTMNG_ERR_NOT_SUPPORTED     (VTMNG_ERRCODE(VS_REASON_NOT_SUPPORTED))    /* requested function not currently supported */
#define VTMNG_ERR_INVALID_SIZE      (VTMNG_ERRCODE(VS_REASON_INVALID_SIZE))     /* size indication out of acceptable range */
#define VTMNG_ERR_INVALID_TYPE      (VTMNG_ERRCODE(VS_REASON_INVALID_TYPE))     /* invalid type encountered */
#define VTMNG_ERR_SOCKET            (VTMNG_ERRCODE(VS_REASON_SOCKET))           /* communication socket reported error */
#define VTMNG_ERR_STATE_EVENT       (VTMNG_ERRCODE(VS_REASON_STATE_EVENT))      /* state/event error */
#define VTMNG_ERR_DOES_NOT_EXIST    (VTMNG_ERRCODE(VS_REASON_DOES_NOT_EXIST))   /* item indicated does not exist */
#define VTMNG_ERR_POLL              (VTMNG_ERRCODE(VS_REASON_POLL))             /* failure during poll operation */
#define VTMNG_ERR_LIST              (VTMNG_ERRCODE(VS_REASON_LIST))             /* list-related error */

#define VTMNG_WRN_TIMEOUT           (VTMNG_ERRCODE(VS_REASON_TIMEOUT))          /* idle timeout period elapsed without activity */
#define VTMNG_WRN_DISCONNECT        (VTMNG_ERRCODE(VS_REASON_DISCONNECT))       /* error due to connection disconnect */
#define VTMNG_WRN_FULL              (VTMNG_ERRCODE(VS_REASON_FULL))             /* currently full */
#define VTMNG_WRN_INVALID_CONFIG    (VTMNG_ERRCODE(VS_REASON_INVALID_CONFIG))   /* invalid configuration entry encountered */
#define VTMNG_WRN_UNEXPECTED        (VTMNG_ERRCODE(VS_REASON_UNEXPECTED))       /* item of known type encountered when given type not expected */

/* ---------------- DCI Definition ------------------- */
#define MAX_DCI_SIZE   256   /* Maximum DCI length */

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * VTP (video transcoder platform)
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * VTP Configuration (Get|Set|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Vtp_Cfg
{
    S8                  vtpName[VTMNG_LONGNAME_SZ]; /* name of this VTP instance [@|EMPTY = use hostname] */
    S8                  vtpDesc[VTMNG_DESC_SZ];     /* VTP description (usually version/revision, etc.) [@|EMPTY = trc_agent-generated] */
    U8                  vtpEvent;                   /* optional event to issue to the top-level VTP controller (VTMNG_VTP_E_xxx) */
    U8                  vtpInitState;               /* set whether VTP will init to a disabled state (VTMNG_VTP_I_xxx) */
    U8                  rtcpMode;                   /* set whether channels act as RTCP translators by default [VTMNG_CFG_DISABLED|VTMNG_CFG_ENABLED] */
    U8                  decodePartials;             /* set whether partial frames should be passed to the decoder [VTMNG_CFG_DISABLED|VTMNG_CFG_ENABLED] */
    U8                  overlayExclusive;           /* whether VTP is reserved for channels requiring overlays [VTMNG_CFG_DISABLED|VTMNG_CFG_ENABLED] */
    U8                  logToConsole;               /* whether "to-file" logging should be forked to console [VTMNG_CFG_DISABLED|VTMNG_CFG_ENABLED] */
    U8                  avail2[3];
    U32                 licenseHighWater;           /* percentage of licenses in use at which time VTMNG_EVENT_VTP_LICENSE notif issued */
    U32                 licenseLowWater;            /* percentage of licenses in use at which time VTMNG_EVENT_VTP_LICENSE notif issued */
    U32                 usageHighWater;             /* percentage of estimated usage at which time VTMNG_EVENT_VTP_USAGE notif issued */
    U32                 usageLowWater;              /* percentage of estimated usage at which time VTMNG_EVENT_VTP_USAGE notif issued */
    U32                 spxMaxChans;                /* maximum number of simplex transcoding channels to allow [2-<total port licenses>] (must be even) */
    S32                 trcpCount;                  /* number of transcoder processes to create [+ = TRCP count [2-spxMaxChans]; - = "simplex channels per TRCP" [1-spxMaxChans] */
    S8                  mediaAddress[VTMNG_LONGNAME_SZ];    /* IP address used for media endpoint access to VTP [@|EMPTY = use same IP address for control and media] */
    U32                 maxRtpPayload;              /* maximum size of any outbound RTP packet payload */
    U32                 apiTimeout;                 /* TRC API watchdog timeout (time allowed for TRC API response) [in msecs] (0 = infinite) */
    U32                 initTimeout;                /* time (after connect) to wait for INIT REQ from TRC API [in msecs] (0 = infinite) */
    U32                 appLostTimeout;             /* time (after disconnect) before considering app lost [in msecs] (0 = infinite) */
    U32                 debugLogMask;               /* global debug log mask (set of VSLOG_xxx bits) */
    U32                 trcpLogMask;                /* global trcp debug log mask (set of VSLOG_xxx bits) */
    U32                 rtcpInTimeout;              /* default RTCP idle (no RTP or RTCP RX) input endpoint timeout [in msecs] */
    U32                 rtcpOutTimeout;             /* default RTCP idle (no RTCP RX) output endpoint timeout [in msecs] */
    U32                 trapMask;                   /* mask of all event types VTP will issue traps for (VTMNG_EVENT_xxx) */
    S8                  trapAddress[VTMNG_LONGNAME_SZ];     /* IP address that all asynchronous indications (traps) are issued to [@|EMPTY = do not issue any traps;& = use requester's address] */
    U32                 trapPort;                   /* UDP port number that async traps are issued to (0 = use well-known [VT_MANAGE_NOTIF_PORT]) */
    U8					codecValidationLevel;		/* type of validation to perform on incoming bitstreams */
    U32					codecDebugMask;				/* bitmask of debugging levels at the codec level */
    S32                 SystemLogFileMaxSize;       /* maximum size of log files in KB 
                                                       (-1 = no limit, file will grow infinitely) */
    S32                 SystemLogFileMaxNum;        /* maximum # of rollovers per log file 
                                                       (-1 = no limit on # of rollovers (no rollovers will be deleted), 
                                                         0 = no rollovers maintained (only active files exist)) */
	U8                  CPUCalcSamples;             /*number of samples to calculate average CPU usage*/
	U32                 rejectHighWater;            /*CPU usage level to start rejecting calls */
	U32                 rejectLowWater;             /*CPU usage level to start re-accepting calls once rejectHighWater limit was reached */
    U8                  displayTimeSrc;             /* source for timestamps [0 = rtp timestamp; 1 = temporal reference normalized to 90KHz; 2 = srt]  */
    U8                  reserved[16];               /* reserved for future use */
} VTMNG_VTP_CFG;

/*---------------------------------------------------------------------------*
 * VTP Status (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Vtp_Status
{
    U8                  vtpState;               /* current overall state (VTMNG_VTP_S_xxx) */
    U8                  avail[3];
    VSLOG_TIME          vtpStartTime;           /* time when VTP server started */
    U32                 vtpLicensedChannels;    /* number of channel (port) licenses that have been obtained */
    U32                 vtpLicensedOverlays;    /* indication of whether VTP is licensed for overlays */
    U8                  reserved[16];           /* reserved for future use */
} VTMNG_VTP_STATUS;

/*---------------------------------------------------------------------------*
 * VTP Statistics (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Vtp_Stats
{
    U32                 errors;         /* count of errors encountered */
    U32                 warnings;       /* count of warnings encountered */
    U32                 appCount;       /* number of video applications connected to server */
    U32                 usageLevel;     /* current usage level of the VTP (expressed as percent) */
    U32                 usedLicenses;   /* number of port licenses that are currently in use */
    U32                 spxInUse;       /* current number of simplex channels in use */
    U32                 fdxInUse;       /* current number of full-duplex channels in use */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_VTP_STATS;

/*---------------------------------------------------------------------------*
 * VTP Entity (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Vtp_Entity
{
    VTMNG_VTP_CFG       cfg;        /* VTP top-level configuration */
    VTMNG_VTP_STATUS    status;     /* current VTP top-level status */
    VTMNG_VTP_STATS     stats;      /* top-level statistics */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_VTP_ENTITY;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * APP (application connection control)
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * APPLICATION Configuration (Get|Set|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_App_Cfg
{
    /* configuration elements that are set internally */
    U32         appUnique;                      /* unique value ID'ing this application connection */
    S8          appName[VTMNG_NAME_SZ];         /* name of application */
    S8          appHost[VTMNG_LONGNAME_SZ];     /* name of host that application executes on */
    U8          reserved[16];                   /* reserved for future use */
} VTMNG_APP_CFG;

/*---------------------------------------------------------------------------*
 * APPLICATION Status (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_App_Status
{
    U8              appState;                   /* current application connection state (VTMNG_APP_S_xxx) */
    U8              avail[3];
    VSLOG_TIME      appStartTime;               /* time when application first connected */
    U8              reserved[16];               /* reserved for future use */
} VTMNG_APP_STATUS;

/*---------------------------------------------------------------------------*
 * APPLICATION Statistics (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_App_Stats
{
    U32             appSpxInUse;                /* current number of simplex channels in use by application */
    U32             appFdxInUse;                /* current number of full-duplex channels in use by application */
    U8              reserved[16];               /* reserved for future use */
} VTMNG_APP_STATS;

/*---------------------------------------------------------------------------*
 * APPLICATION Entity (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_App_Entity
{
    VTMNG_APP_CFG       cfg;        /* application connection configuration */
    VTMNG_APP_STATUS    status;     /* current application connection status */
    VTMNG_APP_STATS     stats;      /* set of statistics maintained per application connection */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_APP_ENTITY;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * MON (monitored process control)
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * MONITORED PROCESS Variable Definition:
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Mon_Var
{
    S8          varName[VTMNG_NAME_SZ];         /* name of optional variable */
    S8          varString[VTMNG_NAME_SZ];       /* string value of variable */
    U8          reserved[8];                    /* reserved for future use */
} VTMNG_MON_VAR;

/*---------------------------------------------------------------------------*
 * MONITORED PROCESS Configuration (Get|Set|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Mon_Cfg
{
    /* configuration elements that are set internally */
    U32         monUnique;                      /* unique value ID'ing this monitored process */
    S8          monName[VTMNG_NAME_SZ];         /* name of monitored process */
    U8          monEvent;                       /* optional event to issue to the process monitor (VTMNG_MON_E_xxx) */
    U8          monInitState;                   /* initial monitoring state for given process (VTMNG_MON_I_xxx) */
    U8          monLostAction;                  /* action to be taken if process is ever lost (VTMNG_MON_A_xxx) */
    S8          monExecutable[VTMNG_NAME_SZ];   /* name of executable file for this process */
    S8          monCmdLine[VTMNG_LONGNAME_SZ];  /* command-line options provided on process creation */
    VTMNG_MON_VAR   monVar[VTMNG_MON_VARS];     /* optional set variables that can be stored by process monitor */
    U8          reserved[16];                   /* reserved for future use */
} VTMNG_MON_CFG;

/*---------------------------------------------------------------------------*
 * MONITORED PROCESS Status (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Mon_Status
{
    U8          monState;               /* current application connection state (VTMNG_MON_S_xxx) */
    U8          avail[3];
    U32         monParent;              /* non-zero value when process parent is also being monitored */
    VSLOG_TIME  monStartTime;           /* time when this process was last started (when under monitor control) */
    U8          reserved[16];           /* reserved for future use */
} VTMNG_MON_STATUS;

/*---------------------------------------------------------------------------*
 * MONITORED PROCESS Statistics (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Mon_Stats
{
    U32             monLost;            /* number of times that monitored process was lost */
    U32             monLostReason;      /* last reason that monitored process was considered lost */
    U8              reserved[16];       /* reserved for future use */
} VTMNG_MON_STATS;

/*---------------------------------------------------------------------------*
 * MONITORED PROCESS Entity (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Mon_Entity
{
    VTMNG_MON_CFG       cfg;            /* application connection configuration */
    VTMNG_MON_STATUS    status;         /* current application connection status */
    VTMNG_MON_STATS     stats;          /* set of statistics maintained per application connection */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_MON_ENTITY;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * Statistics [Histogram] Sub-Structures
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * Statistics Sub-Structure: Data Communication Related stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_St_Comm
{
    U32             frames;             /* frame count */
    U32             packets;            /* packet count */
    U32             bytes;              /* byte count */
    U32             errTooSmall;        /* errors due to packet size less than minimum */
    U32             errAddrChange;      /* errors due to detected change of remote address */
    U32             errOutOfRange;      /* errors due to packets with sequence numbers or timestamps out of current valid range */
    U32             errPartialFrame;    /* errors due to partial frames (frames with incomplete information) */
    U32             errDupSeqNo;        /* errors due to duplicate sequence number detected */
    U32             errMultipleLast;    /* errors due to multiple packets (of same frame) indicating last in frame */
    U32             errInvalidMode;     /* errors due to invalid mode */
    U32             errUnknownType;     /* errors due to unknown packet type detected */
    U32             errOutOfOrder;      /* errors due to packets in non-sequencial order */
    U32             errDataLoss;        /* errors due to loss of content in data stream */
    U32             errTooBig;          /* errors due to frame size growing too big */
    U32             errNoTranscode;     /* errors indicating when decoding a frame did not result in an encoded frame */
    U32             errors;             /* number of errors detected which are outside errXxxx statistics set */
    U32             lastError;          /* last error code counted in errors */
    U8              reserved[16];       /* reserved for future use */
} VTMNG_ST_COMM;

/*---------------------------------------------------------------------------*
 * Statistics Sub-Structure: Video-type related stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_St_VType
{
    U32             iframes;            /* count of I-FRAMEs */
    U32             iframeBytes;        /* total number of I-FRAME bytes */
    U32             pframes;            /* count of P-FRAMEs */
    U32             pframeBytes;        /* total number of P-FRAME bytes */
    U32             leadPframe;         /* number of P-FRAMEs before initial I-FRAME */
    U32             waitTooShort;       /* number of times data processing postponed due to insufficient frame bits */
    U32             waitNoStart;        /* number of times data processing postponed due to lack of start code */
    U32             skipNoAlign;        /* number of times data bits skipped due to lack of alignment code */
    U32             skipBadAlign;       /* number of times data bits skipped due to bad alignment */
    U32             dropEarly;          /* number of times data dropped (arrived early [before other required info available]) */
    U32             dropGarbage;        /* number of times data dropped due to garbage detected */
    U32             dropTooBig;         /* number of frames dropped due to data overflow */
    U32             errors;             /* number of errors encountered */
    U32             lastError;          /* last error encountered */
    U8              reserved[16];       /* reserved for future use */
} VTMNG_ST_VTYPE;

/*---------------------------------------------------------------------------*
 * Statistics Sub-Structure: Per-Direction
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_St_Dir
{
    VTMNG_ST_COMM       rtp;            /* RTP statistics */
    VTMNG_ST_VTYPE      vtype;          /* Video-type related statistics */
    VTMNG_ST_COMM       rtcp;           /* RTCP statistics */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_ST_DIR;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * Addressing Information
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * RTCP-specific Addressing Information
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Rtcp_Info
{
    S8              cname[VTMNG_NAME_SZ];   /* RTCP: CNAME */
    S8              name[VTMNG_NAME_SZ];    /* RTCP: NAME */
    S8              email[VTMNG_NAME_SZ];   /* RTCP: EMAIL */
    S8              phone[VTMNG_NAME_SZ];   /* RTCP: PHONE */
    S8              loc[VTMNG_NAME_SZ];     /* RTCP: LOC */
    S8              tool[VTMNG_NAME_SZ];    /* RTCP: TOOL */
    S8              note[VTMNG_NAME_SZ];    /* RTCP: NOTE */
    U8              reserved[16];               /* reserved for future use */
} VTMNG_RTCP_INFO;

/*---------------------------------------------------------------------------*
 * Full Addressing Information
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Addr_Info
{
    S8              ipAddr[VTMNG_NAME_SZ];          /* IP address */
    U32             udpPort;                        /* UDP port number */
    U32             rtpSsrc;                        /* RTP/RTCP synchronization source */
    VTMNG_RTCP_INFO rtcp;                           /* RTCP-specific information */
    U8              reserved[16];                   /* reserved for future use */
} VTMNG_ADDR_INFO;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * CHN (transcoder channel)
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * CHANNEL Configuration (Get|Set|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Cfg
{
    /* configuration elements that can be set by external manager */
    U8              chnEvent;               /* optional event to issue to the channel (VTMNG_CHN_E_xxx) */
    U8              avail[3];
    U32             chnDebugMask;           /* channel-specific debug log mask (set of VSLOG_xxx bits) */

    /* configuration elements that are set internally */
    U32             appUnique;              /* unique value ID'ing the owning application (same as APP's appUnique) */
    U32             chnUnique;              /* unique value ID'ing this channel */
    S8              chnName[VTMNG_NAME_SZ]; /* optional name applied to channel (assigned by controlling application) */
    U16             chnBasicType;           /* type of channel (specified when channel created) (TRC_CH_SIMPLEX|TRC_CH_FDX) */
    U16             chnRequire;             /* channel feature requirements (TRC_CH_OVERLAY,TRC_CH_RTCP) */
    tTrcEndpoint    endpointA;              /* description of endpoint A configuration */
    tTrcEndpoint    endpointB;              /* description of endpoint B configuration */
    U32             trcpUnique;             /* unique value ID'ing the transcoder process in use by this channel */
                                            /* NOTE: trcp-specific log filename = xc_log_<trcpUnique> (as %02d) */
    U32             trcpProcess;            /* process ID of the transcoder process in use by this channel */
    U32             trcpDemux;              /* value used to demux this channel's messages from trcp connection */
    U8              reserved[16];           /* reserved for future use */
} VTMNG_CHN_CFG;

/*---------------------------------------------------------------------------*
 * CHANNEL Status (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Status
{
    U8              chnState;               /* current state of channel (VTMNG_CHN_S_xxx) */
    U8              avail[3];
    VSLOG_TIME      chnStartTime;           /* time when channel was last started */
    VSLOG_TIME      chnStopTime;            /* time when channel was last stopped */
    VTMNG_ADDR_INFO remTxEndA;              /* information that is known about the remote transmitter (endpoint A) */
    VTMNG_ADDR_INFO remRxEndB;              /* information that is known about the remote receiver (endpoint B) */
    VTMNG_ADDR_INFO remTxEndB;              /* [full-duplex only] info known about remote transmitter (endpoint B) */
    VTMNG_ADDR_INFO remRxEndA;              /* [full duplex only] info known about remote receiver (endpoint A) */
    U32             locRxEndA;              /* UDP port number transcoder receives RTP video stream (from endpoint A) */
    U32             locTxEndB;              /* UDP port number transcoder transmits RTP video stream from (to endpoint B) */
    U32             locRxEndB;              /* [full-duplex only] UDP port number transcoder receives RTP video stream (from endpoint B) */
    U32             locTxEndA;              /* [full duplex only] UDP port number transcoder transmits RTP video stream from (to endpoint A) */
    S8              locIpAddr[VTMNG_NAME_SZ];   /* IP address of the local transcoder side of the RTP/RTCP sessions */
    U8              reserved[16];           /* reserved for future use */
} VTMNG_CHN_STATUS;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics Sub-Structure: RTP direction (rx or tx) stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Rtp_RxTx
{
    VTMNG_ST_COMM       comm;               /* RTP data communication statistics */
    U8                  reserved[16];       /* reserved for future use */
} VTMNG_CHN_RTP_RXTX;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics Sub-Structure: RTP stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Rtp
{
    VTMNG_CHN_RTP_RXTX  rx;             /* RTP receive stats */
    VTMNG_CHN_RTP_RXTX  tx;             /* RTP transmit stats */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_CHN_RTP;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics Sub-Structure: RTCP (rx or tx) stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Rtcp_RxTx
{
    /* sender info */
    U32             ntpMostSig;         /* most significant 32-bits of NTP timestamp */
    U32             ntpLeastSig;        /* least significant 32-bits of NTP timestamp */
    U32             rtpTimestamp;       /* RTP timestamp associated with NTP timestamp */
    U32             senderPktCount;     /* total number of packets sent (as reported by the sender) */
    U32             senderOctetCount;   /* total number of payload bytes sent [no headers/padding] (as reported by the sender) */

    /* reception report block */
    U8              fractionLost;       /* fraction of RTP data packets lost since previous SR or RR was sent */
    U8              avail1[3];
    U32             cumulativePktsLost; /* total number of RTP data packets that have been lost */
    U32             extendedHighSeqNo;  /* low 16-bits = highest sequence number received; high 16-bits = count of cycles */
    U32             interarrivalJitter; /* estimate of statistical variance of RTP data packet interarrival time */
    U32             lastSr;             /* (LSR) middle 32-bits of NTP timestamp from most recently received SR */
    U32             delaySinceLastSr;   /* delay between receiving last SR and sending given reception report block */

    /* RTCP packet counters */
    U32             rtcpSr;             /* sender report packet count */
    U32             rtcpRr;             /* receiver report packet count */
    U32             rtcpSdes;           /* source descriptor packet count */
    U32             rtcpApp;            /* app-specific packet count */
    U32             rtcpBye;            /* bye packet count */

    /* error handling */
    U32             rtcpErrors;         /* number of errors encountered */
    U32             rtcpLastError;      /* last error encountered */

    U8              reserved[16];       /* reserved for future use */
} VTMNG_CHN_RTCP_RXTX;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics Sub-Structure: RTCP endpoint stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Rctp
{
    VTMNG_CHN_RTCP_RXTX rx;             /* common receive stats */
    VTMNG_CHN_RTCP_RXTX tx;             /* common transmit stats */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_CHN_RTCPE;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics Sub-Structure: video type-specific direction (rx or tx) stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_VType_RxTx
{
    VTMNG_ST_VTYPE  vtcomm;             /* video type-specific communication stats */
    U8              reserved[16];       /* reserved for future use */
} VTMNG_CHN_VTYPE_RXTX;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics Sub-Structure: video type-specific endpoint stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_VType
{
    VTMNG_CHN_VTYPE_RXTX    rx;         /* common receive stats */
    VTMNG_CHN_VTYPE_RXTX    tx;         /* common transmit stats */
    U8                      reserved[16];   /* reserved for future use */
} VTMNG_CHN_VTYPE;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics Sub-Structure: endpoint stats
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_End
{
    VTMNG_CHN_RTP   rtp;                /* UDP-level stats */
    VTMNG_CHN_VTYPE vtype;              /* video type-specific stats */
    VTMNG_CHN_RTCPE rtcpInput;          /* RTCP stats for input half of RTCP Translator */
    VTMNG_CHN_RTCPE rtcpOutput;         /* RTCP stats for output half of RTCP Translator */
    U8              reserved[16];       /* reserved for future use */
} VTMNG_CHN_END;

/*---------------------------------------------------------------------------*
 * CHANNEL Statistics (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Stats
{
    VTMNG_CHN_END   endpointA;          /* statistics maintained for endpoint A */
    VTMNG_CHN_END   endpointB;          /* statistics maintained for endpoint B */
    U8              reserved[16];       /* reserved for future use */
} VTMNG_CHN_STATS;

/*---------------------------------------------------------------------------*
 * CHANNEL Entity (Get|Zero):
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_Chn_Entity
{
    VTMNG_CHN_CFG       cfg;        /* channel configuration */
    VTMNG_CHN_STATUS    status;     /* current channel status information */
    VTMNG_CHN_STATS     stats;      /* set of statistics maintained by channel */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_CHN_ENTITY;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * Common Statistics Entry (Total | CurrMin | PerMin | PerHHr)
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * Common Statistics Entry
 *---------------------------------------------------------------------------*/
typedef struct __vtMng_St_Entry
{
    VTMNG_VTP_STATS     vtp;            /* VTP-level statistics */
    VTMNG_ST_DIR        rx;             /* receive statistics */
    VTMNG_ST_DIR        tx;             /* transmit statistics */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_ST_ENTRY;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * Management Message Sub-Structures
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * Common Management message header in use for all message types
 *---------------------------------------------------------------------------*/
typedef struct __vtMngHdr
{
    U8          version;        /* VT management protocol version in use (VTMNG_VERSION) */
    U8          revision;       /* VT management protocol revision in use (VTMNG_REVISION) */
    U16         len;            /* byte length of entire message [sizeof(VTMNG_HDR) + payload] */
    U8          msgOp;          /* message operation (VTMNG_OP_xxx) */
    U8          msgTypeId;      /* message type ID (VTMNG_xxx_ID) */
    U8          msgCategory;    /* message category (VTMNG_CATEG_xxx) */
    U8          avail[1];       /* available for future use */
    U32         senderID;       /* ID set by sender (Req) and copied by responder (Rsp); set by sender (Ind) */
    U32         sequenceNum;    /* outbound sequence number [set by sender] (incremented each time a message is sent) */

    U8          reserved[16];   /* reserved for future use */
} VTMNG_HDR;

/*---------------------------------------------------------------------------*
 * Object ID - information used to identify any managed object
 *---------------------------------------------------------------------------*/
typedef struct __vtMngObj_Id
{
    U32         entUnique;              /* unique value ID'ing this entity; unique across set of entities in a given list:
                                         *   VTP: NOT USED
                                         *   APP: entUnique = cfg.appUnique
                                         *   CHN: entUnique = cfg.chnUnique
                                         * STATS: entUnique = 1..VTMNG_MAX_MIN_BUCKET | 1..VTMNG_MAX_HHR_BUCKET
                                         * TRAPS: entUnique = RESERVED FOR FUTURE USE */

    U8          reserved[4];            /* reserved for future use */
} VTMNG_OBJ_ID;

/*---------------------------------------------------------------------------*
 * entity ID - fully ID's a given entity [unique ID and ASCII name (when avail)]
 *---------------------------------------------------------------------------*/
typedef struct __vtMngEnt_Id
{
    VTMNG_OBJ_ID    entObj;                 /* basic entity object identifier */
    S8              entName[VTMNG_NAME_SZ]; /* optional ASCII name of entity */

    U8              reserved[4];            /* reserved for future use */
} VTMNG_ENT_ID;

/*---------------------------------------------------------------------------*
 * common response information
 *---------------------------------------------------------------------------*/
typedef struct __vtMngRsp_Info
{
    U32         result;                     /* result of request */
    U32         resultCount;                /* number of elements provided in response (when result = 0) */

    U8          reserved[16];               /* reserved for future use */
} VTMNG_RSP_INFO;

/*---------------------------------------------------------------------------*
 * common notification information
 *---------------------------------------------------------------------------*/
typedef struct __vtMngNotif_Info
{
    U32         eventId;                    /* unique value identifying event being notified (VTMNG_EVENT_xxx) */
    U32         errCode;                    /* 0 = successful event; else error code associated with event */
    U32         severity;                   /* severity level of notification (VSLOG_SEV_xxx) */
    U32         options;                    /* options associated with the event|alarm (VSLOG_EVT_xxx) */
    U32         alarmState;                 /* state associated with alarm condition (VSLOG_ALARM_xxx) */
    S8          locName[VTMNG_NAME_SZ];     /* OPTIONAL name of location where notif generated */
    S8          notifDesc[VTMNG_DESC_SZ];   /* OPTIONAL text description of notification */

    U8          reserved[20];               /* reserved for future use */
} VTMNG_NOTIF_INFO;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * Management Messages
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * general request
 *---------------------------------------------------------------------------*/
typedef struct __vtMngReq_Msg
{
    VTMNG_HDR           mngHdr;     /* common message header */
    VTMNG_OBJ_ID        mngObj;     /* destination object */
} VTMNG_REQ_MSG;

/*---------------------------------------------------------------------------*
 * general response
 *---------------------------------------------------------------------------*/
typedef struct __vtMngRsp_Msg
{
    VTMNG_HDR           mngHdr;     /* common message header */
    VTMNG_OBJ_ID        mngObj;     /* destination object from corresponding request */

    VTMNG_RSP_INFO      mngRsp;     /* common response information */
} VTMNG_RSP_MSG;

/*---------------------------------------------------------------------------*
 * general notification
 *---------------------------------------------------------------------------*/
typedef struct __vtMngNotif_Msg
{
    VTMNG_HDR           mngHdr;     /* common message header */
    VTMNG_OBJ_ID        mngObj;     /* source object */

    VTMNG_NOTIF_INFO    info;       /* common notification information */

} VTMNG_NOTIF_MSG;

/*---------------------------------------------------------------------------*
 * get list response
 *---------------------------------------------------------------------------*/
typedef struct __vtMngGetList_Rsp
{
    VTMNG_RSP_MSG       common;         /* common portion */

    VTMNG_ENT_ID        gListEnt[1];    /* first entity in response list (actual count = resultCount) */
} VTMNG_GETLIST_RSP;

/*---------------------------------------------------------------------------*
 * get entity response
 *---------------------------------------------------------------------------*/
typedef struct __vtMngGetEnt_Rsp
{
    VTMNG_RSP_MSG       common;         /* common portion */

    union
    {
        VTMNG_VTP_ENTITY    vtp;        /* VTP top-level entity */
        VTMNG_APP_ENTITY    app;        /* application entity */
        VTMNG_MON_ENTITY    mon;        /* monitored process entity */
        VTMNG_CHN_ENTITY    chn;        /* channel entity */
        VTMNG_ST_ENTRY      stats;      /* statistics */
    } u;

} VTMNG_GETENT_RSP;

/*---------------------------------------------------------------------------*
 * set config request
 *---------------------------------------------------------------------------*/
typedef struct __vtMngSetCfg_Req
{
    VTMNG_REQ_MSG       common;         /* common portion */

    union
    {
        VTMNG_VTP_CFG   vtp;            /* VTP top-level configuration */
        VTMNG_APP_CFG   app;            /* application entity configuration */
        VTMNG_MON_CFG   mon;            /* monitored process entity configuration */
        VTMNG_CHN_CFG   chn;            /* channel entity configuration */
    } value;

    union
    {
        VTMNG_VTP_CFG   vtp;            /* VTP top-level configuration */
        VTMNG_APP_CFG   app;            /* application entity configuration */
        VTMNG_MON_CFG   mon;            /* monitored process entity configuration */
        VTMNG_CHN_CFG   chn;            /* channel entity configuration */
    } mask;

} VTMNG_SETCFG_REQ;

/*---------------------------------------------------------------------------*
 * set config response
 *---------------------------------------------------------------------------*/
typedef struct __vtMngSetCfg_Rsp
{
    VTMNG_RSP_MSG       common;         /* common portion */

    union
    {
        VTMNG_VTP_CFG   vtp;            /* VTP top-level configuration */
        VTMNG_APP_CFG   app;            /* application entity configuration */
        VTMNG_MON_CFG   mon;            /* monitored process entity configuration */
        VTMNG_CHN_CFG   chn;            /* channel entity configuration */
    } u;

} VTMNG_SETCFG_RSP;

/*---------------------------------------------------------------------------*
 * zero statistics response
 *---------------------------------------------------------------------------*/
typedef struct __vtMngZeroStats_Rsp
{
    VTMNG_RSP_MSG       common;         /* common portion */

    union
    {
        VTMNG_VTP_STATS     vtp;        /* VTP top-level statistics (before being zero'd) */
        VTMNG_APP_STATS     app;        /* application statistics (before being zero'd) */
        VTMNG_MON_STATS     mon;        /* monitored process statistics (before being zero'd) */
        VTMNG_CHN_STATS     chn;        /* channel statistics (before being zero'd) */
        VTMNG_ST_ENTRY      stats;      /* total statistics (before being zero'd) */
    } u;

} VTMNG_ZEROSTATS_RSP;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * Management Notification Sub-Structures
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*---------------------------------------------------------------------------*
 * VTP threshold level crossed notification [LICENSE|USAGE]
 *---------------------------------------------------------------------------*/
typedef struct __vtMngVtpLvl_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    U32                 currLevel;      /* current level */
    U32                 prevLevel;      /* previous level */
    U32                 usedLicenses;   /* number of port license currently in use */
    U32                 spxInUse;       /* number of simplex channels currently in use */
    U32                 fdxInUse;       /* number of full-duplex channels currently in use */
    U8                  reserved[16];   /* reserved for future use */

} VTMNG_VTPLVL_NOTIF;

/*---------------------------------------------------------------------------*
 * Application connected (or disconnected) notification [CONNECT|DISCON]
 *---------------------------------------------------------------------------*/
typedef struct __vtMngAppCon_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    VTMNG_APP_ENTITY    app;            /* application information */
    U8                  reserved[16];   /* reserved for future use */

} VTMNG_APPCON_NOTIF;

/*---------------------------------------------------------------------------*
 * Monitired Process lost or recovered notification [LOST|RECOVER]
 *---------------------------------------------------------------------------*/
typedef struct __vtMngMonProc_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    VTMNG_MON_ENTITY    mon;            /* monitored process information */
    U8                  reserved[16];   /* reserved for future use */

} VTMNG_MONPROC_NOTIF;

/*---------------------------------------------------------------------------*
 * CHANNEL Notification: Channel Created
 *---------------------------------------------------------------------------*/
typedef struct __vtMngChnCreate_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    U32                 appUnique;      /* unique value ID'ing the owning application (same as APP's appUnique) */
    U32                 chnUnique;      /* unique value ID'ing this channel */
    S8                  chnName[VTMNG_NAME_SZ]; /* optional name applied to channel (assigned by controlling application) */
    U16                 chnBasicType;   /* basic channel type (TRC_CH_SIMPLEX|TRC_CH_FDX) */
    U16                 chnRequire;     /* channel feature requirements (TRC_CH_OVERLAY,TRC_CH_RTCP) */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_CHN_CREATE_NOTIF;

/*---------------------------------------------------------------------------*
 * CHANNEL Notification: Channel Started
 *---------------------------------------------------------------------------*/
typedef struct __vtMngChnStart_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    VTMNG_CHN_CFG       cfg;            /* channel configuration */
    VTMNG_CHN_STATUS    status;         /* current status information [RTP addressing not yet known] */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_CHN_START_NOTIF;

/*---------------------------------------------------------------------------*
 * CHANNEL Notification: Channel Stopped
 *---------------------------------------------------------------------------*/
typedef struct __vtMngChnStop_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    VTMNG_CHN_CFG       cfg;            /* channel configuration */
    VTMNG_CHN_STATUS    status;         /* status information at time when channel stopped */
    VTMNG_CHN_STATS     stats;          /* set of statistics maintained by channel */
    U8                  reserved[16];   /* reserved for future use */
} VTMNG_CHN_STOP_NOTIF;

/*---------------------------------------------------------------------------*
 * CHANNEL Notification: Channel Destroyed (Dead)
 *---------------------------------------------------------------------------*/
typedef struct __vtMngChnDead_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    U8                  reserved[16];   /* reserved for future use */
} VTMNG_CHN_DEAD_NOTIF;

/*---------------------------------------------------------------------------*
 * CHANNEL Notification: Channel Detected Error
 *---------------------------------------------------------------------------*/
typedef struct __vtMngChnError_Notif
{
    VTMNG_NOTIF_MSG     common;         /* common portion */

    VTMNG_CHN_STATS     stats;          /* set of statistics maintained by channel */
} VTMNG_CHN_ERROR_NOTIF;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * Management Message Groups
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

typedef union
{
    VTMNG_REQ_MSG       common;         /* common portion */

    VTMNG_REQ_MSG       getList;
    VTMNG_REQ_MSG       getEnt;
    VTMNG_SETCFG_REQ    setCfg;
    VTMNG_REQ_MSG       zeroStats;
} VTMNG_REQ_SET;

typedef union
{
    VTMNG_RSP_MSG       common;         /* common portion */

    VTMNG_GETLIST_RSP   getList;
    VTMNG_GETENT_RSP    getEnt;
    VTMNG_SETCFG_RSP    setCfg;
    VTMNG_ZEROSTATS_RSP zeroStats;
} VTMNG_RSP_SET;

typedef union
{
    VTMNG_NOTIF_MSG         common;     /* common portion */

    VTMNG_VTPLVL_NOTIF      vtpLvl;     /* VTP events related to level change [threshold crossing] */
    VTMNG_NOTIF_MSG         vtpError;   /* VTP-level error detected */
    VTMNG_APPCON_NOTIF      appCon;     /* application connection-related events */
    VTMNG_MONPROC_NOTIF     monProc;    /* monitored process-related events */
    VTMNG_CHN_CREATE_NOTIF  chnCreate;  /* channel created event */
    VTMNG_CHN_START_NOTIF   chnStart;   /* channel started event */
    VTMNG_CHN_STOP_NOTIF    chnStop;    /* channel stopped event */
    VTMNG_CHN_DEAD_NOTIF    chnDead;    /* channel destroyed (dead) event */
    VTMNG_CHN_ERROR_NOTIF   chnError;   /* channel detected error event */
} VTMNG_NOTIF_SET;

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * VTP Management Message
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

typedef union
{
    VTMNG_HDR       hdr;        /* header that is common to all message types */

    VTMNG_REQ_SET   req;        /* request */
    VTMNG_RSP_SET   rsp;        /* response */
    VTMNG_NOTIF_SET notif;      /* notification */
} VT_MNG_MSG;

#endif /* _TRANSMANAGE_H */
