/*****************************************************************************
 *
 * vserr.h - Vision Server error code definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *                      Copyright (c) 1988-2007
 *                by NMS Communications Corporation.
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of NMS Communications Corporation.
 *
 *****************************************************************************/

#ifndef _VSERR_H
#define _VSERR_H

#define VS_ERRCODE_ID           0xE             /* ID used in <id> nibble of every error code */

/*****************************************************************************
 * Registered SYSTEMS
 *
 * (used for error codes as well as other value types)
 *****************************************************************************/
#define VS_SYSTEM_CORE          0x01            /* common Vision Server core [treated as SYSTEM for BASE] */
#define VS_SYSTEM_VIDEO_XCODER  0x02            /* Video Transcoder-specific error code base */

/*****************************************************************************
 * Vision Server Core SYSTEM (VS_SYSTEM_CORE): Base Values
 *
 * (used for error codes as well as other value types)
 *****************************************************************************/

/* ----- VS_SYSTEM_CORE: subsystem base definitions ----- */
#define VS_SYSTEM_BASE          0x01            /* Vision Server system-wide base */
#define VS_OSCORE_BASE          0x02            /* OS core execution abstraction layer */
#define VS_LIST_BASE            0x03            /* linked list manipulation */
#define VS_TIMER_BASE           0x04            /* timer control */
#define VS_FSM_BASE             0x05            /* Finite State Machines */
#define VS_LOG_BASE             0x06            /* logging (system and debug) */
#define VS_PARSE_BASE           0x07            /* text parsing */
#define VS_REMOTE_BASE          0x08            /* remote control */
#define VS_TCP_BASE             0x09            /* TCP connection control */
#define VS_UDP_BASE             0x0A            /* UDP communication control */
#define VS_RTCP_BASE            0x0B            /* RTCP communication control */
#define VS_RTP_BASE             0x0C            /* RTP communication control */
#define VS_MEM_BASE             0x0D            /* MEMORY control */
#define VS_POOL_BASE            0x0E            /* POOLED MEMORY control */

/*****************************************************************************
 * Error code definition macro
 *
 * Error code nibbles: [n7][n6] [n5][n4] [n3][n2] [n1][n0]
 *                    <rsv><id> <system> <base..> <unique>
 *
 * NOTE: High order bit of <unique> indicates that remainder of unique
 *       provides a common reason (VS_REASON_xxx).
 *****************************************************************************/
#define VS_ERROR_CODE(system,base,unique) ((VS_ERRCODE_ID<<24)|(system<<16)|(base<<8)|unique)
#define VS_ERR_REASON       (1<<7)              /* set when remainder of <unique> holds VS_REASON_xxx code */
#define VS_ERR_REASON_MASK  0x000000FF          /* mask to apply to get common reason code */
#define VS_ERR_REASON_SHIFT 0                   /* bits to shift to get common reason code */

/*****************************************************************************
 * Vision Server system-wide error codes
 *
 * (used for generic error indications that are not sub-system specific)
 *****************************************************************************/
#define VSSYS_ERRCODE(unique) VS_ERROR_CODE(VS_SYSTEM_CORE,VS_SYSTEM_BASE,unique)

#define VS_SUCCESS      0                       /* general Vision Server success indication */
#define VS_IN_PROGRESS  (VSSYS_ERRCODE(0x01))   /* special error code indicating that request is in progress */

/*****************************************************************************
 * Vision Server system-wide reason codes
 *
 * (used for generic error indications with a common reason)
 *****************************************************************************/
#define VS_REASON_GENERAL               0       /* general error code - not mapped to a common reason */
#define VS_REASON_INVALID_STATE         0x80    /* invalid state for requested operation */
#define VS_REASON_INVALID_TYPE          0x81    /* invalid type encountered */
#define VS_REASON_NOT_IMPLEMENTED       0x82    /* requested function not yet implemented */
#define VS_REASON_INVALID_CONFIG        0x83    /* configuration entry is invalid */
#define VS_REASON_MISSING_CONFIG        0x84    /* required parameter missing from configuration */
#define VS_REASON_WRONG_THREAD          0x85    /* code executing from different thread than expected */
#define VS_REASON_FAILED_WRITE          0x86    /* error while attempting write */
#define VS_REASON_INVALID_SIZE          0x87    /* size indication out of acceptable range range */
#define VS_REASON_INVALID_KEY           0x88    /* invalid key (handle) detected */
#define VS_REASON_PARTIAL_READ          0x89    /* read of partial buffer (truncated read) */
#define VS_REASON_OUT_OF_MEMORY         0x8A    /* insufficient memory to complete request */
#define VS_REASON_FULL                  0x8B    /* full indication - unable to add more */
#define VS_REASON_TIMER                 0x8C    /* timer-related error */
#define VS_REASON_LIST                  0x8D    /* list-related error */
#define VS_REASON_STATE_EVENT           0x8E    /* state/event error */
#define VS_REASON_PIPE                  0x8F    /* communication pipe-related error */
#define VS_REASON_SET                   0x90    /* error while performing set operation */
#define VS_REASON_GET                   0x91    /* error while performing get operation */
#define VS_REASON_REG                   0x92    /* unable to complete registration */
#define VS_REASON_POLL                  0x93    /* failure during poll operation */
#define VS_REASON_NULL                  0x94    /* NULL value encountered when valid pointer expected */
#define VS_REASON_INVALID_HDR           0x95    /* invalid header record encountered */
#define VS_REASON_INVALID_INDEX         0x96    /* index value out of range */
#define VS_REASON_INVALID_ADDRESS       0x97    /* invalid address provided */
#define VS_REASON_UNALIGNED_DATA        0x98    /* data area with address that is not of required alignment */
#define VS_REASON_INVALID_PKT           0x99    /* packet encountered fails validation checks */
#define VS_REASON_FAILED_CREATE         0x9A    /* failed to create required component */
#define VS_REASON_SOCKET                0x9B    /* communication socket reported error */
#define VS_REASON_CONNECT               0x9C    /* error while attempting to connect */
#define VS_REASON_ACCEPT                0x9D    /* error when trying to accept a remote connection */
#define VS_REASON_WAIT_OBJECT           0x9E    /* error related to OS-specific wait object */
#define VS_REASON_INSTALL_HANDLER       0x9F    /* unable to install handler function */
#define VS_REASON_CANCELLED             0xA0    /* operation has been cancelled */
#define VS_REASON_EXISTS                0xA1    /* the item indicated already exists */
#define VS_REASON_DOES_NOT_EXIST        0xA2    /* the item indicated does not exist */
#define VS_REASON_NOT_SUPPORTED         0xA3    /* requested function not currently supported */
#define VS_REASON_NOT_INITIALIZED       0xA4    /* module has not been initialized */
#define VS_REASON_INVALID_PARAM         0xA5    /* invalid input parameter */
#define VS_REASON_IGNORED               0xA6    /* request has been ignored */
#define VS_REASON_EMPTY                 0xA7    /* selection set is currently empty */
#define VS_REASON_NOT_NUMERIC           0xA8    /* value must be a numeric value */
#define VS_REASON_OUT_OF_RANGE          0xA9    /* numeric value out of valid range */
#define VS_REASON_MISSING_PARAMS        0xAA    /* missing reqired parameter(s) */
#define VS_REASON_FAILED_OPEN           0xAB    /* error while opening */
#define VS_REASON_FAILED_READ           0xAC    /* error while reading */
#define VS_REASON_TIMEOUT               0xAD    /* operation timed out */
#define VS_REASON_EXIT                  0xAE    /* terminated since process exit requested */
#define VS_REASON_DISCONNECT            0xAF    /* error due to connection disconnect */
#define VS_REASON_NO_RESOURCE           0xB0    /* unable to obtain required resource */
#define VS_REASON_VERSION               0xB1    /* unsupported version indicated */
#define VS_REASON_REVISION              0xB2    /* future revision encountered */
#define VS_REASON_INVALID_FILE          0xB3    /* invalid file format encountered */
#define VS_REASON_INVALID_FUNC          0xB4    /* invalid pointer to the callback function encountered */
#define VS_REASON_ALREADY               0xB5    /* function requested has already been performed */
#define VS_REASON_INVALID_NAME          0xB6    /* invalid name encountered */
#define VS_REASON_PROCESS_LOST          0xB7    /* indication that required process has been lost */
#define VS_REASON_UNEXPECTED            0xB8    /* item of known type encountered when given type not expected */
#define VS_REASON_ABORTED               0xB9    /* given operation has been aborted */
#define VS_REASON_RESTART               0xBA    /* failure due to process restart request */
#define VS_REASON_REBOOT                0xBB    /* failure due to system reboot request */

#endif  /* _VSERR_H */
