
/*****************************************************************************
 * trcsample.c - Sample Application showing usage of transcoder (TRC) API
 *
 * This sample application shows how to use the TRC API to control a set of
 * transcoder resources. Transcoder resources are provided by external servers
 * called VTPs (Video Transcoder Platforms).
 *****************************************************************************/

/*****************************************************************************
 **
 **                         PROPRIETARY NOTICE
 **
 **                       Copyright (c) 2000-2007
 **                 By NMS Communications Corporation.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of NMS Communications Corporation.
 **
 *****************************************************************************/

/*****************************************************************************
 * Sample Application Overview:
 *
 * Working examples of all TRC API functions are provided by trcsample:
 *
 * ===========================================================================
 * TRC API Function        Usage by Sample Application
 * ----------------------- ---------------------------------------------------
 * trcSetTrace             sets API tracing level to log all errors
 * trcInitialize           initializes the API (configuring IP addrs of VTPs)
 *
 * trcCreateVideoChannel   used to create simplex and full-duplex channels
 * trcNameVideoChannel     assigns a name to a channel
 * trcInfoVideoChannel     used to obtain addressing information assigned to channels
 * trcStartVideoChannel    used to configure and start various channel types
 * trcIframeVideoChannel   causes an IFRAME to be sent by the transcoder
 * trcStopVideoChannel     used to stop any channel type
 * trcDestroyVideoChannel  used to destroy channels
 *
 * trcCreateOverlay        Create text or image ovrlays
 * trcStartOverlay         Start an inactive overlay
 * trcStopOverlay          Stop an active overlay
 * trcDestroyOverlay       Destroy an overlay
 *
 * trcUsage                provides top-level transcoder resource usage information
 * trcVTPStatus            provides current connection status of all VTPs
 * trcChannelStatus        provides current channel status for all channels owned locally
 *
 * trcResetVTP             used to reset a VTP (causing the server to reboot)
 *
 * trcShutdown             cleanly closes down the API
 *
 * trcValueName            helper function providing ASCII names for various TRC value codes
 *
 * ===========================================================================
 * Command Line Syntax: trcsample [ -c <apiConfigFile> ]
 *                                [ -s <VTPtoShow> ]
 *                                [ -r <VTPtoReset> ]
 *
 * <apiConfigFile>
 *   The application provides the list of VTPs to the TRC API using a
 *   configuration file. For the trcsample application this filename can be
 *   specified on the command line (as <apiConfigFile>) or left to the default
 *   filename: UNIX: /opt/nms/trc/trcapi.cfg
 *             WINDOWS: c:\nms\trc\trcapi.cfg
 *
 *   NOTE: Whether using the default configuration file or a filename provided
 *         on the command line, the file must have been modified to include
 *         accurate IP addresses for all VTPs to be controlled by the TRC API.
 *
 * <VTPtoShow>
 *   The sample application shows detailed status information instead of
 *   performing channel example connections.
 *
 *   NOTE: VTP numbers are 1-based
 *
 * <VTPtoReset>
 *   The sample application does not call trcResetVTP by default since this
 *   causes the entire VTP to reboot, which should not be done under normal
 *   usage of the TRC API. Use the -r option to cause trcsample to reset the
 *   specified VTP instead of performing channel example connections.
 *
 *****************************************************************************/

/*****************************************************************************
 * Detailed Description of trcsample Application:
 * ==============================================
 *
 * Execution Overview:
 * -------------------
 * The sample application operates in one of 3 modes.
 *
 * If -s <VTPtoShow> is specified, the sample application requests and
 * displays detailed status information for the given VTP.
 * NOTE: The TRC API is written to provide exclusive control of transcoder
 *       resources to the owning application. If another application has
 *       exclusive control (i.e. has issued trcInitialize) then the sample
 *       application will not be able to connect to the VTPs and will
 *       therefore not display accurate status information for the given VTP.
 *
 * If -r <VTPtoReset> is specified, the sample application resets (reboots)
 * the specified VTP.
 *
 * If neither -s nor -r is specified, the sample application works through
 * a set of example connections with each connection using the following
 * call flow:
 *
 *   <CREATE CHANNEL>
 *          v
 *   <START CHANNEL>........(wait START DONE)
 *          v
 *   [<CONFIG MPEG4s>.......(wait CONFIG DONE)]
 *          v
 *   <GENERATE I-FRAMEs>....(wait CONFIG DONE)
 *          v
 *   <STOP CHANNEL>.........(wait STOP DONE)
 *          v
 *   <DESTROY CHANNEL>
 *
 * Application Thread Model
 * ------------------------
 * The trcsample application operates as a single thread. Multi-threaded
 * applications are fully supported by the TRC API; but for the sake of
 * simplicity, all trcsample code executes from a single thread
 * (except for the callback function).
 *
 * When the sample application is used to reset a VTP, there is no need for
 * a callback function. For the channel example case, however, asynchronous
 * notifications are provided by the TRC API in the form of a direct upcall
 * to a callback function provided by the application. It is the application's
 * responsibility to provide a thread-safe mechanism for switching control
 * back to the application's thread before processing these notifications.
 *
 * A more complete video control application should use a scheme involving
 * a pipe mechanism or writing to / reading from Natural Access queues
 * as a means of safely passing indications from the callback function
 * to the target application thread. This would allow an application to
 * maintain several different channels, with each channel at a different stage
 * within its call flow. For the sample application, a more simplified method
 * of switching threads is used. The sample application uses a single semaphore
 * to control switching between the threads with a global 'lastMsg' holding the
 * last asynchronous event provided by the TRC API through the callback
 * function. Because the sample application never has more than 1 asynchronous
 * notification that it is waiting on, this single semaphore / single message
 * is an adequate control mechanism.
 *
 * Lack of Connection Control Protocol
 * -----------------------------------
 * Identifying video endpoints that are to be connected with video transcoding
 * is beyond the scope of this sample application. For a complete video
 * application, incoming calls from video endpoints must be connected with
 * outgoing video endpoints. The NMS Video Access product provides the
 * additional functionality needed to manange external endpoint connections.
 *
 * Addressing for External Video Endpoints
 * ---------------------------------------
 * Since the trcsample application does not include any external connection
 * management layer, it has no way of obtaining valid IP addresses for external
 * video endpoints. The sample will use the IP address of the VTP being used
 * for transcoding. This allows the examples to operate without connecting to
 * any real endpoints, but would never be used for real implementations.
 * In a real implementation, external endpoint addresses would be determined
 * as part of the call setup phase.
 *
 * Types of Channels Shown as Examples
 * -----------------------------------
 * Separate examples are shown for each of the following types of connections:
 *
 *   SIMPLEX connection from H.263 to MPEG4
 *
 *   SIMPLEX connection from MPEG4 to H.263
 *
 *   SIMPLEX connection between two H.263 endpoints
 *
 *   SIMPLEX connection between two MPEG4 endpoints
 *
 *   FULL-DUPLEX connection between H.263 and MPEG4 endpoints
 *
 *   FULL-DUPLEX connection between MPEG4 and H.263 endpoints
 *
 *   FULL-DUPLEX connection between two H.263 endpoints
 *
 *   FULL-DUPLEX connection between two MPEG4 endpoints
 *
 * NOTE: For FULL-DUPLEX connections, it is an arbitrary decision whether to
 *       represent a given leg of the connection as leg1 (params1) or
 *       leg2 (params2). There is no difference between the FULL-DUPLEX
 *       connection examples of H.263 to MPEG4 vs. MPEG4 to H.263.
 *       Both of these examples result in a channel that is transcoding
 *       in both directions between the two video formats.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef UNIX
#include <semaphore.h>
#include <unistd.h>
#else
#include <windows.h>
#endif

#include "trcapi.h"

#ifdef UNIX
typedef sem_t           tSampleSemaphore;
#else
typedef HANDLE          tSampleSemaphore;
#endif

/* SLEEP macro = sleep for # of seconds specified */
#ifdef UNIX
#define SLEEP(x) sleep(x)
#else
#define SLEEP(x) Sleep(x*1000)
#endif

#define TRC_FAILURE 1

/*###########################################################################*
 * GLOBALS
 *###########################################################################*/
tTrcVtpAll          currTrcStatus;      /* holds last status information requested from TRC API */
U32                 sleepCnt = 0;       /* number of times that the sample app has slept awaiting VTP connections */
tTrcMessage         lastMsg;            /* holds last notification message received from API */
tSampleSemaphore    sampleSemaphore;    /* semaphore used to synchonize API calls against TRC thread */
int                 dummyPort = 90000;  /* source of unique dummy port numbers */
int                 testNum = 0;        /* test number currently executing (0 = not test yet) */
int                 testTotal = 9;      /* total number of test connections performed */
U8                  debug = 0;          /* 0 = quiet mode; else print additional debug messages */
/* dummy record to be used for providing MPEG4 decoder DCI configuration */
char            sampleMpeg4ConfigDCI[] =
{ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
#define SAMPLE_MPEG4_CONFIG_SZ 10

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 * PROTOTYPES
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* ----- semaphore control functions (abstracting OS-dependent calls) ----- */
int sampleCreateSemaphore   ( tSampleSemaphore *semaphore );
int sampleWaitSemaphore     ( tSampleSemaphore *semaphore );
int sampleReleaseSemaphore  ( tSampleSemaphore *semaphore );
int sampleDestroySemaphore  ( tSampleSemaphore *semaphore );

/*###########################################################################*
 * INTERNAL FUNCTIONS
 *###########################################################################*/

/*****************************************************************************
 * sampleCreateChannel - sub-function used to create any of the example channels
 *
 * This function issues a CREATE CHANNEL request and then yeilds control
 * until the CREATE CHANNEL DONE asynchronous notification is received
 * by the upcall function (sampleEventHandler).
 * Once the upcall has occurred, control is switched back to this function.
 *****************************************************************************/
int sampleCreateChannel( TRC_HANDLE *chHandle, U16 type, S8 *testName )
{
    U32         result;
    int         rc;
    tTrcChInfo  chInfo;

    /* ----- create a channel ----- */
    result = trcCreateVideoChannel( chHandle, type, NULL );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcCreateVideoChannel\n",
                trcValueName(TRCVALUE_RESULT, result) );
        result = TRC_FAILURE;
    }
    else
    {
        if (debug)
        {
            printf( "    CREATE %s CHANNEL... in progress (awaiting CREATE DONE event)\n",
                    trcValueName( TRCVALUE_TYPE, type ) );
        }
        rc = sampleWaitSemaphore( &sampleSemaphore );
        if (rc != 0)
        {
            printf( "!!! ERROR: %d waiting on semaphore\n", rc );
            result = TRC_FAILURE;
        }
        else
        {
            if ( (lastMsg.event == TRCEVN_CREATE_CHANNEL_DONE) &&
                 (lastMsg.result == TRC_SUCCESS) )
            {
                if (debug)
                {
                    printf( "    ...CREATE CHANNEL completed successfully\n" );
                }

                result = trcInfoVideoChannel( *chHandle, &chInfo );
                if (result == TRC_SUCCESS)
                {
                    if (debug)
                    {
                        printf( "    CHANNEL name (assigned by TRC API): %s\n", chInfo.name );
                        printf( "    CHANNEL ID   (assigned by VTP)    : 0x%08X\n", chInfo.summary.vtpChanId );
                    }
                }
                else
                {
                    printf( "!!! ERROR: %s returned from trcInfoVideoChannel\n",
                            trcValueName( TRCVALUE_RESULT, result ) );
                    result = TRC_FAILURE;
                }

                /* set the name of the channel
                 * NOTE: A channel's name can be altered at any time after the channel has been created.
                 *       There is no need to name a channel, but naming channels can help manager
                 *       applications more easily identify particular channels.
                 */
                result = trcNameVideoChannel( *chHandle, testName );
                if (result != TRC_SUCCESS)
                {
                    printf( "!!! ERROR: %s returned from trcNameVideoChannel\n",
                            trcValueName( TRCVALUE_RESULT, result ) );
                    result = TRC_FAILURE;
                }

                if (type == TRC_CH_SIMPLEX)
                {   /* simplex channel: endpoint A outputs to transcoder; transcoder outputs to endpoint B */
                    if (debug)
                    {
                        printf( "    Endpoint (A) should send video bitstream to address:port %s:%d\n",
                                chInfo.res.ipAddr, chInfo.res.rxPort[TRC_DIR_SIMPLEX] );
                    }
                }
                else
                {   /* full-duplex channel: both endpoint A and endpoint B send output to transcoder */
                    if (debug)
                    {
                        printf( "    Endpoint (A) should send video bitstream to address:port %s:%d\n",
                                chInfo.res.ipAddr, chInfo.res.rxPort[TRC_DIR_FDX1] );
                        printf( "    Endpoint (B) should send video bitstream to address:port %s:%d\n",
                                chInfo.res.ipAddr, chInfo.res.rxPort[TRC_DIR_FDX2] );
                    }
                }
            }
            else
            {
                printf( "!!! ERROR: Unexpected event [%s] or result [%s] awaiting CREATE CHANNEL\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result) );
                result = TRC_FAILURE;
            }
        }
    }
    return( result );
}

/*****************************************************************************
 * sampleStartChannel - sub-function used to start any of the example channels
 *
 * This function issues a START CHANNEL request and then yeilds control
 * until the START CHANNEL DONE asynchronous notification is received
 * by the upcall function (sampleEventHandler).
 * Once the upcall has occurred, control is switched back to this function.
 *****************************************************************************/
int sampleStartChannel( TRC_HANDLE chHandle, tTrcChConfig *chConfig )
{
    U32     result;
    int     rc;

    /* configure and start the channel */
    result = trcStartVideoChannel( chHandle, chConfig );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcStartVideoChannel\n",
                trcValueName( TRCVALUE_RESULT, result ) );
    }
    else
    {   /* wait for the API to issue START DONE notification */

        /* Use the semaphore to put the sample application to sleep until
         * the upcall notification has occurred indicating that the START
         * has completed. See sampleEventHandler for code that posts to
         * the semaphore (causing sampleWaitSemaphore to complete).
         */
        if (debug)
        {
            printf( "    START CHANNEL... in progress (awaiting START DONE event)\n" );
        }
        rc = sampleWaitSemaphore( &sampleSemaphore );
        if (rc != 0)
        {
            printf( "!!! ERROR: %d waiting on semaphore\n", rc );
            result = TRC_FAILURE;
        }
        else
        {   /* semaphore has been signalled due to notification from API */

            /* verify that the notification received is the one that was expected */
            if ( (lastMsg.event == TRCEVN_START_CHANNEL_DONE) &&
                 (lastMsg.result == TRC_SUCCESS) )
            {
                if (debug)
                {
                    printf( "    ...START CHANNEL completed successfully\n" );
                }
            }
            else
            {
                printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting START DONE\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result));
                result = TRC_FAILURE;
            }
        }
    }

    return( result );
}

/*****************************************************************************
 * sampleStopChannel - sub-function used to stop any of the example channels
 *
 * This function issues a STOP CHANNEL request and then yeilds control
 * until the STOP CHANNEL DONE asynchronous notification is received
 * by the upcall function (sampleEventHandler).
 * Once the upcall has occurred, control is switched back to this function.
 *****************************************************************************/
int sampleStopChannel( TRC_HANDLE chHandle )
{
    U32     result;
    int     rc;

    /* stop the channel */
    result = trcStopVideoChannel( chHandle );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcStopVideoChannel\n",
                trcValueName(TRCVALUE_RESULT, result) );
    }
    else
    {   /* wait for the API to issue STOP DONE notification */

        /* Use the semaphore to put the sample application to sleep until
         * the upcall notification has occurred indicating that the STOP
         * has completed. See sampleEventHandler for code that posts to
         * the semaphore (causing sampleWaitSemaphore to complete).
         */
        if (debug)
        {
            printf( "    STOP CHANNEL in progress (awaiting STOP DONE event...)\n" );
        }
        rc = sampleWaitSemaphore( &sampleSemaphore );
        if (rc != 0)
        {
            printf( "!!! ERROR: %d waiting on semaphore\n", rc );
            result = TRC_FAILURE;
        }
        else
        {   /* semaphore has been signalled due to notification from API */

            /* verify that the notification received is the one that was expected */
            if ( (lastMsg.event == TRCEVN_STOP_CHANNEL_DONE) &&
                 (lastMsg.result == TRC_SUCCESS) )
            {
                if (debug)
                {
                    printf( "    ...STOP CHANNEL completed successfully\n" );
                }
            }
            else
            {
                printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting STOP DONE\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result));
                result = TRC_FAILURE;
            }
        }
    }

    return( result );
}

/*****************************************************************************
 * sampleGenIframe - sub-function used to generate I-FRAME out any channel
 *
 * This function issues a CONFIG CHANNEL request and then yeilds control
 * until the CONFIG CHANNEL DONE asynchronous notification is received
 * by the upcall function (sampleEventHandler).
 * Once the upcall has occurred, control is switched back to this function.
 *****************************************************************************/
int sampleGenIframe( TRC_HANDLE chHandle, U16 direction )
{
    U32     result;
    int     rc;

    /* cause the channel to generate I-FRAME out direction indicated */
    result = trcIframeVideoChannel( chHandle, direction );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcIframeVideoChannel\n",
                trcValueName(TRCVALUE_RESULT, result) );
    }
    else
    {   /* wait for the API to issue IFRAME DONE notification */

        /* Use the semaphore to put the sample application to sleep until
         * the upcall notification has occurred indicating that the IFRAME
         * has been generated. See sampleEventHandler for code that posts to
         * the semaphore (causing sampleWaitSemaphore to complete).
         */
        if (debug)
        {
            printf( "    GENERATE IFRAME in progress [%s direction] (awaiting IFRAME DONE event...)\n",
                    trcValueName( TRCVALUE_DIRECTION, direction ) );
        }
        rc = sampleWaitSemaphore( &sampleSemaphore );
        if (rc != 0)
        {
            printf( "!!! ERROR: %d waiting on semaphore\n", rc );
            result = TRC_FAILURE;
        }
        else
        {   /* semaphore has been signalled due to notification from API */

            /* verify that the notification received is the one that was expected */
            if ( (lastMsg.event == TRCEVN_IFRAME_CHANNEL_DONE) &&
                 (lastMsg.result == TRC_SUCCESS) )
            {
                if (debug)
                {
                    printf( "    ...GENERATE IFRAME completed successfully [%s direction]\n",
                            trcValueName( TRCVALUE_DIRECTION, lastMsg.data[TRCDATA_IFRAME_CHANNEL_DIRECTION] ) );
                }
            }
            else
            {
                printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting IFRAME DONE\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result));
                result = TRC_FAILURE;
            }
        }
    }

    return( result );
}

/********************************************************************************
 * sampleCreateTextOverlay - sub-function used to generate a text overlay.
 ********************************************************************************/
int sampleCreateTextOverlay( TRC_HANDLE chHandle, U16 direction, TRC_OVL_HANDLE *ovlHandle, TRC_OVL_USERKEY ovlUserKey )
{

    tTrcOvlConfig   ovlConfig;
    tTrcOvlContent  ovlContent;
    U32 result;
    U32 eventWaitCount;

    memset(&ovlConfig, 0, sizeof(ovlConfig));
    memset(&ovlContent, 0, sizeof(ovlContent));

    ovlConfig.type              = TRC_OVL_TYPE_SINGLELINE_TEXT;
    ovlConfig.initState         = TRC_OVL_INITSTATE_STARTED;  /* This overlay iscreated in the started state */
    ovlConfig.layer             = 1;
    ovlConfig.size.xUnit        = TRC_OVL_COORDINATE_UNIT_PERCENT;
    ovlConfig.size.x            = 80;   /* Overlay width is 80 percent of the frame width */
    ovlConfig.size.yUnit        = TRC_OVL_COORDINATE_UNIT_PERCENT;
    ovlConfig.size.y            = 10;   /* Overlay height is 10 percent of the frame height*/
    ovlConfig.position.xUnit    = TRC_OVL_COORDINATE_UNIT_PIXEL;
    ovlConfig.position.x        = 10;   /* Top left corner of overlay is at 10 pixels horizontally from top left of frame */
    ovlConfig.position.yUnit    = TRC_OVL_COORDINATE_UNIT_PIXEL;
    ovlConfig.position.y        = 20;   /* Top left corner of overlay is at 10 pixels vertically from top left of frame */
    ovlConfig.bgColor           = TRC_OVL_COLOR_TRANSPARENT;
    ovlConfig.border.width      = 1;    /* A one pixel blue border around overlay */
    ovlConfig.border.color      = TRC_OVL_COLOR_BLUE;

    strcpy(ovlConfig.font.name, "arial"); /* Use cached font (see definition in [fonts] section on trcr.cfg) */
    ovlConfig.font.size         = 16;
    ovlConfig.font.style        = TRC_OVL_FONTSTYLE_OUTLINED;           /* Outline around text */
    TRC_OVL_SET_RGB(ovlConfig.font.fgColor, 0xFF, 0xFF, 0xFF, 0xFF);    /* Same as TRC_OVL_COLOR_WHITE */
    TRC_OVL_SET_RGB(ovlConfig.font.bgColor, 0x88, 0x88, 0x88, 0x50);    /* Translucent grey background */

    ovlContent.type             = TRC_OVL_CONT_TYPE_TEXT;
    ovlContent.options          = TRC_OVL_TEXT_CONT_ALIGN_LEFT;
    ovlContent.scroll.type      = TRC_OVL_SCROLL_TYPE_NONE;
    ovlContent.ovlData          = "Text overlay sample";

    result = trcCreateOverlay( chHandle, TRC_DIR_SIMPLEX, &ovlConfig, &ovlContent, ovlUserKey);
    if (result == TRC_SUCCESS)
    {
        eventWaitCount = 0; /* We're waiting for TRCEVN_CREATE_OVL_DONE and TRCEVN_CHANNEL_OVL_EVENT with TRC_OVLEVT_TRCR_RENDER_SUCCESS */
        while (eventWaitCount < 2)
        {
            result = sampleWaitSemaphore( &sampleSemaphore );
            if (result != 0)
            {
                printf( "!!! ERROR: %d waiting on semaphore\n", result );
            }
            else
            {   /* semaphore has been signalled due to notification from API */

                /* verify that the notification received is the one that was expected */
                if ( (lastMsg.event == TRCEVN_CREATE_OVL_DONE) &&
                     (lastMsg.result == TRC_SUCCESS) )
                {
                    *ovlHandle = (TRC_OVL_HANDLE)lastMsg.data[0];
                    if (debug)
                    {
                        printf( "    ...Create Overlay completed successfully\n" );
                    }
                }
                else if ( (lastMsg.event == TRCEVN_CHANNEL_OVL_EVENT) &&
                     (lastMsg.result == TRC_OVLEVT_TRCR_RENDER_SUCCESS) )
                {
                    result = TRC_SUCCESS;
                    if (debug)
                    {
                        printf( "    ...Overlay content sucessfully rendered\n" );
                    }
                }
                else
                {
                    printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting OVERLAY DONE or OVERLAY EVENT: RENDER SUCCESS (1)\n",
                            trcValueName(TRCVALUE_EVENT, lastMsg.event),
                            trcValueName(TRCVALUE_RESULT, lastMsg.result));
                }
                result = lastMsg.result;
            }
            eventWaitCount++;
        }
    }
    else
    {
        printf("!!! ERROR: Unexpected result from trcCreateOverlay() = 0x%08x [%s]\n", result, trcValueName(TRCVALUE_RESULT, result));
    }
    return result;
}

/********************************************************************************
 * sampleCreateImageOverlay - sub-function used to generate an image overlay.
 ********************************************************************************/
int sampleCreateImageOverlay( TRC_HANDLE chHandle, U16 direction, TRC_OVL_HANDLE *ovlHandle, TRC_OVL_USERKEY ovlUserKey )
{

    tTrcOvlConfig   ovlConfig;
    tTrcOvlContent  ovlContent;
    U32 result;
    U32 eventWaitCount;

    memset(&ovlConfig, 0, sizeof(ovlConfig));
    memset(&ovlContent, 0, sizeof(ovlContent));

    ovlConfig.type              = TRC_OVL_TYPE_IMAGE;
    ovlConfig.initState         = TRC_OVL_INITSTATE_STARTED;
    ovlConfig.layer             = 2;
    ovlConfig.size.xUnit        = TRC_OVL_COORDINATE_UNIT_PIXEL;
    ovlConfig.size.x            = 100;   /* Overlay width is 100 pixels of the frame width */
    ovlConfig.size.yUnit        = TRC_OVL_COORDINATE_UNIT_PIXEL;
    ovlConfig.size.y            = 50;   /* Overlay height is 50 pixels of the frame height*/
    ovlConfig.position.xUnit    = TRC_OVL_COORDINATE_UNIT_PERCENT;
    ovlConfig.position.x        = 0;    /* Top left corner of overlay is at 0% horizontally from top left of frame */
    ovlConfig.position.yUnit    = TRC_OVL_COORDINATE_UNIT_PERCENT;
    ovlConfig.position.y        = 50;   /* Top left corner of overlay is at 50% vertically from top left of frame */
    ovlConfig.bgColor           = TRC_OVL_COLOR_TRANSPARENT;
    ovlConfig.border.width      = 0;    /* No border around overlay */
    ovlConfig.border.color      = TRC_OVL_COLOR_DEFAULT;

    ovlContent.ovlData               = "local://NMS_Logo_100x50";
    ovlContent.type                  = TRC_OVL_CONT_TYPE_GRAPHIC;
    ovlContent.options               = TRC_OVL_GRAPH_CONT_CENTER;
    ovlContent.scroll.type           = TRC_OVL_SCROLL_TYPE_CONTINOUS;
    ovlContent.scroll.speedDir.xUnit = TRC_OVL_COORDINATE_UNIT_MS_PER_PIXEL;
    ovlContent.scroll.speedDir.x     = -100; /* Advance one pixel to the left every 100 ms (or 10 pixels per second) */
    ovlContent.scroll.speedDir.yUnit = TRC_OVL_COORDINATE_UNIT_UNUSED;
    ovlContent.scroll.speedDir.y     = 0;

    result = trcCreateOverlay( chHandle, TRC_DIR_SIMPLEX, &ovlConfig, &ovlContent, ovlUserKey);
    if (result == TRC_SUCCESS)
    {
        eventWaitCount = 0; /* We're waiting for TRCEVN_CREATE_OVL_DONE and TRCEVN_CHANNEL_OVL_EVENT with TRC_OVLEVT_TRCR_RENDER_SUCCESS */
        while (eventWaitCount < 2)
        {
            result = sampleWaitSemaphore( &sampleSemaphore );
            if (result != 0)
            {
                printf( "!!! ERROR: %d waiting on semaphore\n", result );
            }
            else
            {   /* semaphore has been signalled due to notification from API */

                /* verify that the notification received is the one that was expected */
                if ( (lastMsg.event == TRCEVN_CREATE_OVL_DONE) &&
                     (lastMsg.result == TRC_SUCCESS) )
                {
                    *ovlHandle = (TRC_OVL_HANDLE)lastMsg.data[0];
                    if (debug)
                    {
                        printf( "    ...Create Overlay completed successfully\n" );
                    }
                }
                else if ( (lastMsg.event == TRCEVN_CHANNEL_OVL_EVENT) &&
                     (lastMsg.result == TRC_OVLEVT_TRCR_RENDER_SUCCESS) )
                {
                    result = TRC_SUCCESS;
                    if (debug)
                    {
                        printf( "    ...Overlay content sucessfully rendered\n" );
                    }
                }
                else
                {
                    printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting OVERLAY DONE or OVERLAY EVENT: RENDER SUCCESS (2)\n",
                            trcValueName(TRCVALUE_EVENT, lastMsg.event),
                            trcValueName(TRCVALUE_RESULT, lastMsg.result));
                }
                result = lastMsg.result;
            }

            eventWaitCount++;
        }
    }
    else
    {
        printf("!!! ERROR: Unexpected result from trcCreateOverlay() = 0x%08x [%s]\n", result, trcValueName(TRCVALUE_RESULT, result));
    }
    return result;
}

/*****************************************************************************
 * sampleStartOverlay - sub-function used to generate a start overlay.
 *****************************************************************************/
int sampleStartOverlay( TRC_HANDLE chHandle, TRC_OVL_HANDLE ovlHandle )
{
    U32 result;

    result = trcStartOverlay( chHandle, ovlHandle);
    if (result == TRC_SUCCESS)
    {
        result = sampleWaitSemaphore( &sampleSemaphore );
        if (result != 0)
        {
            printf( "!!! ERROR: %d waiting on semaphore\n", result );
        }
        else
        {   /* semaphore has been signalled due to notification from API */

            /* verify that the notification received is the one that was expected */
            if ( (lastMsg.event == TRCEVN_START_OVL_DONE) &&
                 (lastMsg.result == TRC_SUCCESS) )
            {
                if (debug)
                {
                    printf( "    ...Start Overlay completed successfully\n" );
                }
            }
            else
            {
                printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting START OVERLAY DONE\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result));
            }
            result = lastMsg.result;

            /* Wait for content to be rendered */
            result = sampleWaitSemaphore( &sampleSemaphore );
            if (result != 0)
            {
                printf( "!!! ERROR: %d waiting on semaphore\n", result );
            }
            else
            {   /* semaphore has been signalled due to notification from API */

                /* verify that the notification received is the one that was expected */
                if ( (lastMsg.event == TRCEVN_CHANNEL_OVL_EVENT) &&
                     (lastMsg.result == TRC_OVLEVT_TRCR_RENDER_SUCCESS) )
                {
                    if (debug)
                    {
                        printf( "    ...Overlay content sucessfully rendered\n" );
                    }
                }
                else
                {
                    printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting OVERLAY EVENT: RENDER SUCCESS\n",
                            trcValueName(TRCVALUE_EVENT, lastMsg.event),
                            trcValueName(TRCVALUE_RESULT, lastMsg.result));
                }
                result = lastMsg.result;
            }
        }
    }
    else
    {
        printf("!!! ERROR: Unexpected result from trcStartOverlay() = 0x%08x [%s]\n", result, trcValueName(TRCVALUE_RESULT, result));
    }
    return result;
}

/*****************************************************************************
 * sampleStopOverlay - sub-function used to generate a stop overlay.
 *****************************************************************************/
int sampleStopOverlay( TRC_HANDLE chHandle, TRC_OVL_HANDLE ovlHandle )
{
    U32 result;

    result = trcStopOverlay( chHandle, ovlHandle);
    if (result == TRC_SUCCESS)
    {
        result = sampleWaitSemaphore( &sampleSemaphore );
        if (result != 0)
        {
            printf( "!!! ERROR: %d waiting on semaphore\n", result );
        }
        else
        {   /* semaphore has been signalled due to notification from API */
            /* verify that the notification received is the one that was expected */
            if ( (lastMsg.event == TRCEVN_STOP_OVL_DONE) &&
                 (lastMsg.result == TRC_SUCCESS) )
            {
                if (debug)
                {
                    printf( "    ...Stop Overlay completed successfully\n" );
                }
            }
            else
            {
                printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting STOP OVERLAY DONE\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result));
            }
            result = lastMsg.result;
        }
    }
    else
    {
        printf("!!! ERROR: Unexpected result from trcStopOverlay() = 0x%08x [%s]\n", result, trcValueName(TRCVALUE_RESULT, result));
    }
    return result;
}

/*****************************************************************************
 * sampleDestroyOverlay - sub-function used to generate a destroy overlay.
 *****************************************************************************/
int sampleDestroyOverlay( TRC_HANDLE chHandle, TRC_OVL_HANDLE ovlHandle )
{
    U32 result;

    result = trcDestroyOverlay( chHandle, ovlHandle);
    if (result == TRC_SUCCESS)
    {
        result = sampleWaitSemaphore( &sampleSemaphore );
        if (result != 0)
        {
            printf( "!!! ERROR: %d waiting on semaphore\n", result );
        }
        else
        {   /* semaphore has been signalled due to notification from API */

            result = lastMsg.result;
            /* verify that the notification received is the one that was expected */
            if ( (lastMsg.event == TRCEVN_DESTROY_OVL_DONE) &&
                 (lastMsg.result == TRC_SUCCESS) )
            {
                if (debug)
                {
                    printf( "    ...Destroy Overlay completed successfully\n" );
                }
            }
            else
            {
                printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting DESTROY OVERLAY DONE\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result));
            }
            result = lastMsg.result;
        }
    }
    else
    {
        printf("!!! ERROR: Unexpected result from trcDestroyOverlay() = 0x%08x [%s]\n", result, trcValueName(TRCVALUE_RESULT, result));
    }
    return result;
}

/********************************************************************************
 * sampleCreateHttpOverlay - sub-function used to generate an overlay
 *                           from an image file retrieved from a web server.
 *
 *                           This example starts an overlay in the stopped state
 *                           so it can start it only once it receives confirmation
 *                           that the content has be successfully rendered.
 *                           This ca be usefull when displaying multiple images
 *                           that all need to be fetched and resized,
 *                           simultaneously.
 ********************************************************************************/
int sampleCreateHttpOverlay( TRC_HANDLE chHandle, U16 direction, TRC_OVL_HANDLE *ovlHandle, TRC_OVL_USERKEY ovlUserKey )
{

    tTrcOvlConfig   ovlConfig;
    tTrcOvlContent  ovlContent;
    U32 result;
    U32 eventWaitCount;

    memset(&ovlConfig, 0, sizeof(ovlConfig));
    memset(&ovlContent, 0, sizeof(ovlContent));

    ovlConfig.type              = TRC_OVL_TYPE_IMAGE;
    ovlConfig.initState         = TRC_OVL_INITSTATE_STOPPED;  /* Overlay created in stopped state. It will started
                                                                 by the application once it receives the
                                                                 TRC_OVLEVT_TRCR_RENDER_SUCCESS event */
    ovlConfig.layer             = 2;
    ovlConfig.size.xUnit        = TRC_OVL_COORDINATE_UNIT_PIXEL;
    ovlConfig.size.x            = 100;   /* Overlay width is 100 pixels of the frame width */
    ovlConfig.size.yUnit        = TRC_OVL_COORDINATE_UNIT_PIXEL;
    ovlConfig.size.y            = 50;   /* Overlay height is 50 pixels of the frame height*/
    ovlConfig.position.xUnit    = TRC_OVL_COORDINATE_UNIT_PERCENT;
    ovlConfig.position.x        = 0;    /* Top left corner of overlay is at 0% horizontally from top left of frame */
    ovlConfig.position.yUnit    = TRC_OVL_COORDINATE_UNIT_PERCENT;
    ovlConfig.position.y        = 50;   /* Top left corner of overlay is at 50% vertically from top left of frame */
    ovlConfig.bgColor           = TRC_OVL_COLOR_TRANSPARENT;
    ovlConfig.border.width      = 0;    /* No border around overlay */
    ovlConfig.border.color      = TRC_OVL_COLOR_DEFAULT;

                                       /* Orange "N" logo from the NMS web site */
    ovlContent.ovlData               = "http://www.nmscommunications.com/NR/rdonlyres/BF255F77-5E83-45C3-B246-1A0752DAE41E/0/NBackToTop.gif";
    ovlContent.type                  = TRC_OVL_CONT_TYPE_GRAPHIC;
    ovlContent.options               = TRC_OVL_GRAPH_CONT_CENTER;
    ovlContent.scroll.type           = TRC_OVL_SCROLL_TYPE_NONE;
    ovlContent.scroll.speedDir.xUnit = TRC_OVL_COORDINATE_UNIT_UNUSED;
    ovlContent.scroll.speedDir.x     = 0;
    ovlContent.scroll.speedDir.yUnit = TRC_OVL_COORDINATE_UNIT_UNUSED;
    ovlContent.scroll.speedDir.y     = 0;

    result = trcCreateOverlay( chHandle, TRC_DIR_SIMPLEX, &ovlConfig, &ovlContent, ovlUserKey);
    if (result == TRC_SUCCESS)
    {
        eventWaitCount = 0; /* We're waiting for TRCEVN_CREATE_OVL_DONE and TRCEVN_CHANNEL_OVL_EVENT with TRC_OVLEVT_TRCR_RENDER_SUCCESS */
        while (eventWaitCount < 2)
        {
            result = sampleWaitSemaphore( &sampleSemaphore );
            if (result != 0)
            {
                printf( "!!! ERROR: %d waiting on semaphore\n", result );
            }
            else
            {   /* semaphore has been signalled due to notification from API */

                /* verify that the notification received is the one that was expected */
                if ( (lastMsg.event == TRCEVN_CREATE_OVL_DONE) &&
                     (lastMsg.result == TRC_SUCCESS) )
                {
                    *ovlHandle = (TRC_OVL_HANDLE)lastMsg.data[0];
                    if (debug)
                    {
                        printf( "    ...Create Overlay completed successfully\n" );
                    }
                }
                else if ( (lastMsg.event == TRCEVN_CHANNEL_OVL_EVENT) &&
                     (lastMsg.result == TRC_OVLEVT_TRCR_RENDER_SUCCESS) )
                {
                    result = TRC_SUCCESS;
                    if (debug)
                    {
                        printf( "    ...Overlay content sucessfully rendered\n" );
                    }
                }
                else
                {
                    printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting OVERLAY DONE or OVERLAY EVENT: RENDER SUCCESS (3)\n",
                            trcValueName(TRCVALUE_EVENT, lastMsg.event),
                            trcValueName(TRCVALUE_RESULT, lastMsg.result));
                }
                result = lastMsg.result;
            }

            if (result != TRC_SUCCESS)
            {
                /* Content is ready, start the text overlay */
                result = sampleStartOverlay(chHandle, *ovlHandle);
            }
            eventWaitCount++;
        }
    }
    else
    {
        printf("!!! ERROR: Unexpected result from trcCreateOverlay() = 0x%08x [%s]\n", result, trcValueName(TRCVALUE_RESULT, result));
    }
    return result;
}

/*****************************************************************************
 * sampleTestOverlays - sub-function used to validate overlay requests.
 *****************************************************************************/
int sampleTestOverlays( TRC_HANDLE chHandle, U16 direction )
{
    U32 result;
    TRC_OVL_HANDLE  txtOvlHandle, imgOvlHandle, httpOvlHandle;

    result = sampleCreateTextOverlay(chHandle, direction, &txtOvlHandle, 0x00000100);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    result = sampleCreateImageOverlay(chHandle, direction, &imgOvlHandle, 0x00000200);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    result = sampleCreateHttpOverlay(chHandle, direction, &httpOvlHandle, 0x00000200);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    /* Stop the text overlay */
    result = sampleStopOverlay(chHandle, txtOvlHandle);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    /* Start the text overlay */
    result = sampleStartOverlay(chHandle, txtOvlHandle);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    /* Stop the image overlay */
    result = sampleStopOverlay(chHandle, txtOvlHandle);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    /* Start the image overlay */
    result = sampleStartOverlay(chHandle, txtOvlHandle);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    result = sampleDestroyOverlay(chHandle, txtOvlHandle);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    result = sampleDestroyOverlay(chHandle, imgOvlHandle);
    if (result != TRC_SUCCESS)
    {
        return result;
    }

    result = sampleDestroyOverlay(chHandle, httpOvlHandle);
    if (result != TRC_SUCCESS)
    {
        return result;
    }
    return result;
}


/*****************************************************************************
 * sampleDestroyChannel - sub-function used to destroy any of the example channels
 *
 * This function issues a DESTROY CHANNEL request and then yeilds control
 * until the DESTORY CHANNEL DONE asynchronous notification is received
 * by the upcall function (sampleEventHandler).
 * Once the upcall has occurred, control is switched back to this function.
 *****************************************************************************/
int sampleDestroyChannel( TRC_HANDLE chHandle )
{
    U32     result;
    int     rc;

    /* ----- create a channel ----- */
    result = trcDestroyVideoChannel( chHandle );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcDestroyVideoChannel\n",
                trcValueName(TRCVALUE_RESULT, result) );
        result = TRC_FAILURE;
    }
    else
    {
        if (debug)
        {
            printf("    DESTROY CHANNEL... in progress (awaiting DESTROY DONE event)\n");
        }
        rc = sampleWaitSemaphore( &sampleSemaphore);
        if ( rc != 0)
        {
            printf("!!! ERROR: %d waiting on semaphore\n", rc);
            result = TRC_FAILURE;
        }
        else
        {
            if ( ( lastMsg.event == TRCEVN_DESTROY_CHANNEL_DONE)  &&
                  (lastMsg.result == TRC_SUCCESS))
            {
                if (debug)
                {
                    printf("    ...DESTROY CHANNEL completed successfully\n");
                }
            }
            else
            {
                printf("!!! ERROR: Unexpected event [%s] or result [%s] awaiting DESTROY CHANNEL\n",
                        trcValueName(TRCVALUE_EVENT, lastMsg.event),
                        trcValueName(TRCVALUE_RESULT, lastMsg.result));
                result = TRC_FAILURE;
            }
        }
    }
    return( result );
}


/*****************************************************************************
 * sampleDetailStatus - display detailed status information for VTP indicated
 *****************************************************************************/
void sampleDetailStatus( int vtpId )
{
    tTrcVtpSummary     *vtp = &currTrcStatus.vtp[vtpId - 1];
    tTrcChAll          chAll;
    tTrcChSummary      *chSum;
    tTrcChStatus       *channel;
    U32                 result;
    int                 i;

    printf( "*** Detailed Status Information for VTP %d:\n", vtpId );
    /* show top-level VTP information */
    printf( "    VTP-%02d (%s) licensesAvail=%d, simplexInUse=%d, duplexInUse=:%d\n",
            vtp->vtpId,
            trcValueName(TRCVALUE_VTPSTATE, vtp->state),
            vtp->usage.licensesAvail,       /* total transcoder port licenses available */
            vtp->usage.simplexInUse,        /* total simplex channels currently in use by all applications sharing the same VTP */
            vtp->usage.fdxInUse );          /* total full duplex channels currently in use by all applications sharing the same VTP */

    /* show information about each non-idle channel */
    if ( (vtp->usage.simplexInUse == 0) &&
         (vtp->usage.fdxInUse == 0) )
    {
        printf( "    No Active Channels on VTP %d\n", vtp->vtpId );
    }
    else
    {
        result = trcChannelStatus(&chAll);
        if (result != TRC_SUCCESS)
        {
            printf( "!!! ERROR: %s returned from trcChannelStatus.\n",
                    trcValueName(TRCVALUE_RESULT, result) );
        }

        for (i = 0; i < chAll.chanDefined; i++)
        {
            chSum = &(chAll.chan[i]);
            channel = &(chSum->status);
            if (chSum->vtpId != vtp->vtpId)
                continue;

            printf( "    %s %s Channel [ID=0x%08X]\n",
                    trcValueName(TRCVALUE_CHSTATE, channel->state),
                    trcValueName(TRCVALUE_TYPE, channel->type),
                    chSum->vtpChanId );
            if (channel->errorDesc.numErrors != 0)
            {
                printf( "    errors: %d, first: %s [OS %d] [%s], last: %s [OS %d] [%s]\n",
                        channel->errorDesc.numErrors,
                        trcValueName( TRCVALUE_RESULT, channel->errorDesc.firstError.trcError ),
                        channel->errorDesc.firstError.osError,
                        channel->errorDesc.firstError.errorInfo,
                        trcValueName( TRCVALUE_RESULT, channel->errorDesc.lastError.trcError ),
                        channel->errorDesc.lastError.osError,
                        channel->errorDesc.lastError.errorInfo );
            }
        }
    }
}

/*****************************************************************************
 * sampleNextDummyPort - assign a unique port number to be used as dummy port
 *
 * Because the sample application includes no actual call control, this
 * function is used to generate unique port numbers for each test.
 * For a real application, endpoint ports are determined as part of the
 * call setup phase.
 *****************************************************************************/
int sampleNextDummyPort( )
{
    dummyPort++;

    return( dummyPort );
}

/*****************************************************************************
 * sampleBanner - display detailed status information for VTP indicated
 *****************************************************************************/
void sampleBanner( char *testDesc )
{
    testNum++;
    printf( "-------------------------------------------------------------------------------\n" );
    printf( "TEST %d of %d: %s\n", testNum, testTotal, testDesc );
    printf( "-------------------------------------------------------------------------------\n" );
}

/*###########################################################################*
 * SEMAPHORE CONTROL FUNCTIONS (abstracting OS-dependent calls)
 *###########################################################################*/

/*****************************************************************************
 * sampleCreateSemaphore - create a semaphore to be used for thread sync
 *****************************************************************************/
int sampleCreateSemaphore( tSampleSemaphore *semaphore )
{
    int     ret = 0;

    if (semaphore == NULL)
        return( -1 );

#ifdef UNIX
    memset( semaphore, 0, sizeof(tSampleSemaphore) );

    ret = sem_init( semaphore, 0, 0 );
	if (ret != 0)
        return( -1 );
#else
    ret = 0;                /* suppress compiler warning */
    *semaphore = CreateSemaphore( NULL, 0, 1, NULL );
    if (*semaphore == NULL)
        return( -1 );

    if(GetLastError() == ERROR_ALREADY_EXISTS)
    {
        CloseHandle( *semaphore );
        return( -1 );
    }
#endif

    return( 0 );
}

/*****************************************************************************
 * sampleWaitSemaphore - wait on a semaphore (suspend execution until release)
 *****************************************************************************/
int sampleWaitSemaphore( tSampleSemaphore *semaphore )
{
    int     ret = 0;

    if (semaphore == NULL)
        return( -1 );

#ifdef UNIX
    ret = sem_wait( semaphore );
#else
    {
        U32 dwWaitResult;

        dwWaitResult = WaitForSingleObject( *semaphore, INFINITE );
        switch (dwWaitResult)
        {
            case WAIT_OBJECT_0:
                break;

            case WAIT_TIMEOUT:
                ret = -1;
                break;

            case WAIT_ABANDONED:
                break;

            default:        /* includes WAIT_FAILED */
                ret = -1;
                break;
        }
    }
#endif

    if (ret != 0)
        return( -1 );

    return( 0 );
}

/*****************************************************************************
 * sampleReleaseSemaphore - release a semaphore (ending suspended execution)
 *****************************************************************************/
int sampleReleaseSemaphore( tSampleSemaphore *semaphore )
{
    if (semaphore == NULL)
        return( -1 );

#ifdef UNIX
    if (sem_post( semaphore ) != 0)
        return( -1 );
#else
    if (ReleaseSemaphore( *semaphore, 1, NULL ) != TRUE)
        return( -1 );
#endif

    return( 0 );
}

/*****************************************************************************
 * sampleDestroySemaphore - destroy a semaphore
 *****************************************************************************/
int sampleDestroySemaphore( tSampleSemaphore *semaphore )
{
    int     ret;

    if (semaphore == NULL)
        return( -1 );

#ifdef UNIX
    ret = sem_destroy( semaphore );
	if (ret != 0)
        return( -1 );
#else
    ret = CloseHandle( *semaphore );
    *semaphore = NULL;
    if (ret != TRUE)
        return( -1 );
#endif

    return( 0 );
}

/*###########################################################################*
 * EXAMPLES
 *###########################################################################*/

/*****************************************************************************
 * sampleSimplexH2M - create a simplex connection (H.263 to MPEG4 endpoints)
 *
 * This example takes a simplex channel through a standard call flow.
 *
 * The example connects an H.263 endpoint to an MPEG4 endpoint outputting
 * at 40 kilobits/sec and 7 frames/sec.
 *
 * This example configures the transcoder to listen for RTCP traffic
 * associated with the inbound RTP stream and to issue RTCP back to the given
 * RTCP source (as appropriate).
 *
 * (Endpoint A [H.263])   ===> |TRANSCODER| ===>   (Endpoint B [MPEG4])
 *****************************************************************************/
void sampleSimplexH2M( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "SIMPLEX channel: H.263 ==> MPEG4" );

    /* ----- create a simplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_SIMPLEX, "sampleSimplexH2M" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );           /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_H263;          /* input H.263 video format */
    chConfig.endpointA.profile = TRC_PROFILE_BASELINE;      /* input H.263 video format */
    chConfig.endpointA.level   = TRC_H263_LEVEL_10;         /* input H.263 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_MPEG4;         /* output MPEG4 video format */
    chConfig.endpointB.profile = TRC_PROFILE_SIMPLE;        /* output MPEG4 video format */
    chConfig.endpointB.level   = TRC_MPEG4_LEVEL_0;         /* output MPEG4 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 40;                      /* input H.263 video at 40 kilobits per second */
    chConfig.endpointA.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    chConfig.endpointB.dataRate = 40;                       /* output MPEG4 video at 40 kilobits per second */
    chConfig.endpointB.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_2190;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_3016;

    /* activate receive of RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointA.chanIn.rtcpRxTimeout = 0;                /* max time (in msecs) that can pass without receiving RTP or RTCP before considering endpoint timed out (0 = no timeout) */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to MPEG4 endpoint address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to MPEG4 endpoint port */
    chConfig.endpointB.chanOut.payloadId = 100;                 /* MPEG4 uses payload IDs from 96 to 127 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointB.chanOut.rtcpTxTimeout = 0;               /* max time (in msecs) that can pass without receiving RTCP before considering endpoint timed out (0 = no timeout) */

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to the MPEG4 endpoint ----- */
    /* NOTE: This step is not necessary as part of channel startup.
     *       A channel may be asked to transmit an I-FRAME at any time after the
     *       channel has been started.
     */
    result = sampleGenIframe( chHandle, TRC_DIR_SIMPLEX );  /* generate I-FRAME out simplex channel */
    /* If an error occurs here, we are stopping and destroying the channel anyways... */

    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleSimplexM2H - create a simplex connection (MPEG4 to H.263 endpoints)
 *
 * This example takes a simplex channel through a standard call flow.
 *
 * The example connects an MPEG4 endpoint to an H.263 endpoint outputting
 * at 40 kilobits/sec and 7 frames/sec.
 *
 * (Endpoint A [MPEG4])   ===> |TRANSCODER| ===>   (Endpoint B [H.263])
 *****************************************************************************/
void sampleSimplexM2H( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "SIMPLEX channel: MPEG4 ==> H.263" );

    /* ----- create a simplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_SIMPLEX, "sampleSimplexM2H" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );           /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_MPEG4;         /* input MPEG4 video format */
    chConfig.endpointA.profile = TRC_PROFILE_SIMPLE;        /* input MPEG4 video format */
    chConfig.endpointA.level   = TRC_MPEG4_LEVEL_0;         /* input MPEG4 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_H263;          /* output H.263 video format */
    chConfig.endpointB.profile = TRC_PROFILE_BASELINE;      /* output H.263 video format */
    chConfig.endpointB.level   = TRC_H263_LEVEL_10;         /* output H.263 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 40;                      /* input H.263 video at 40 kilobits per second */
    chConfig.endpointA.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    chConfig.endpointB.dataRate = 40;                       /* output MPEG4 video at 40 kilobits per second */
    chConfig.endpointB.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_3016;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_2429;

    /* do not listen for RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to MPEG4 endpoint address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to MPEG4 endpoint port */
    chConfig.endpointB.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.duplicateInitialI.isSet = TRC_TRUE;  /* overriding default initial IFRAME duplication */
    chConfig.endpointB.chanOut.duplicateInitialI.value = TRC_FALSE; /* do not duplicate the initial outbound IFRAME */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

    /* configure the simplex decoder that is transcoding from MPEG-4 endpoint A to H.263 endpoint B */
    chConfig.decoder[TRC_DIR_SIMPLEX].optData = sampleMpeg4ConfigDCI;
    chConfig.decoder[TRC_DIR_SIMPLEX].optSize = sizeof(sampleMpeg4ConfigDCI);

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to the H.263 endpoint ----- */
    /* NOTE: This step is not necessary as part of channel startup.
     *       A channel may be asked to transmit an I-FRAME at any time after the
     *       channel has been started.
     */
    result = sampleGenIframe( chHandle, TRC_DIR_SIMPLEX );  /* generate I-FRAME out simplex channel */
    /* If an error occurs here, we are stopping and destroying the channel anyways... */

    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );

}

/*****************************************************************************
 * sampleSimplexH2H - create a simplex connection (both sides H.263 endpoints)
 *
 * This example takes a simplex channel through a standard call flow.
 *
 * Even when both sides of a connection are using the same video encoding type
 * (H.263 in this example), transcoding is still required to convert from
 * one type of frame resolution / data rate / frame rate to another.
 * In this example, an H.263 input data stream using QCIF resolution is
 * connected to an H.263 output data stream outputting CIF.
 *
 * (Endpoint A [QCIF])   ===> |TRANSCODER| ===>   (Endpoint B [CIF])
 *****************************************************************************/
void sampleSimplexH2H( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "SIMPLEX channel: H.263 ==> H.263" );

    /* ----- create a simplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_SIMPLEX, "sampleSimplexH2H" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );           /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_H263;          /* input H.263 video format */
    chConfig.endpointA.profile = TRC_PROFILE_BASELINE;      /* input H.263 video format */
    chConfig.endpointA.level   = TRC_H263_LEVEL_10;         /* input H.263 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_H263;          /* output H.263 video format */
    chConfig.endpointB.profile = TRC_PROFILE_BASELINE;      /* output H.263 video format */
    chConfig.endpointB.level   = TRC_H263_LEVEL_30;         /* output H.263 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 40;                      /* input H.263 video at 40 kilobits per second */
    chConfig.endpointA.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    chConfig.endpointB.dataRate = 200;                      /* output CIF video at 200 kilobits per second */
    chConfig.endpointB.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_CIF;       /* CIF (352 x 288) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_2190;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_2429;

    /* do not listen for RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to CIF endpoint address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to CIF endpoint port */
    chConfig.endpointB.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to the CIF endpoint ----- */
    /* NOTE: This step is not necessary as part of channel startup.
     *       A channel may be asked to transmit an I-FRAME at any time after the
     *       channel has been started.
     */
    result = sampleGenIframe( chHandle, TRC_DIR_SIMPLEX );  /* generate I-FRAME out simplex channel */
    /* If an error occurs here, we are stopping and destroying the channel anyways... */

    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleSimplexM2M - create a simplex connection (both sides MPEG4 endpoints)
 *
 * This example takes a simplex channel through a standard call flow.
 *
 * Even when both sides of a connection are using the same video encoding type
 * (MPEG4 in this example), transcoding is still required to convert from
 * one type of data rate / frame rate to another.
 * In this example, an MPEG4 input data stream (at 200 kbps and
 * 15 fps) is connected to an MPEG4 output data stream with an output
 * data rate of 40 kilobits/sec and output frame rate of 10 frames/sec.
 *
 * (Endpoint A [200 @ 15])   ===> |TRANSCODER| ===>   (Endpoint B [40 @ 10])
 *****************************************************************************/
void sampleSimplexM2M( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "SIMPLEX channel: MPEG4 ==> MPEG4" );

    /* ----- create a simplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_SIMPLEX, "sampleSimplexM2M" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );           /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_MPEG4;         /* input MPEG4 video format */
    chConfig.endpointA.profile = TRC_PROFILE_SIMPLE;        /* input MPEG4 video format */
    chConfig.endpointA.level   = TRC_MPEG4_LEVEL_3;         /* input MPEG4 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_MPEG4;         /* output MPEG4 video format */
    chConfig.endpointB.profile = TRC_PROFILE_SIMPLE;        /* output MPEG4 video format */
    chConfig.endpointB.level   = TRC_MPEG4_LEVEL_1;         /* output MPEG4 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 200;                     /* input H.263 video at 200 kilobits per second */
    chConfig.endpointA.frameRate = 15;                      /* 15 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_CIF;       /* CIF (352 x 288) frame resolution */

    chConfig.endpointB.dataRate  = 40;                      /* output CIF video at 200 kilobits per second */
    chConfig.endpointB.frameRate = 10;                      /* 10 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_3016;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_3016;

    /* do not listen for RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to endpoint B address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to endpoint B port */
    chConfig.endpointB.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to endpoint B ----- */
    /* NOTE: This step is not necessary as part of channel startup.
     *       A channel may be asked to transmit an I-FRAME at any time after the
     *       channel has been started.
     */
    result = sampleGenIframe( chHandle, TRC_DIR_SIMPLEX );  /* generate I-FRAME out simplex channel */
    /* If an error occurs here, we are stopping and destroying the channel anyways... */

    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleFullDuplexH2M - create a full-duplex connection (btw H.263 and MPEG4)
 *
 * This example takes a full-duplex channel through a standard call flow.
 *
 * The example connects an H.263 endpoint (operating at 40 kbps and 7 fps)
 * with an MPEG4 endpoint (operating at 40 kbps and 7 fps)
 *
 * This example configures the transcoder to listen for RTCP traffic
 * associated with the inbound RTP streams and to send associated RTCP
 * along with outbound RTP stream.
 *
 *                      FDX1 ===> |          | ===> FDX1
 * (Endpoint A [H.263])           |TRANSCODER|           (Endpoint B [MPEG4])
 *                      FDX2 <=== |          | <=== FDX2
 *****************************************************************************/
void sampleFullDuplexH2M( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "FULL-DUPLEX channel: H.263 <=> MPEG4" );

    /* ----- create a full-duplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_FDX, "sampleFullDuplexH2M" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );           /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_H263;          /* input H.263 video format */
    chConfig.endpointA.profile = TRC_PROFILE_BASELINE;      /* input H.263 video format */
    chConfig.endpointA.level   = TRC_H263_LEVEL_10;         /* input H.263 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_MPEG4;         /* output MPEG4 video format */
    chConfig.endpointB.profile = TRC_PROFILE_SIMPLE;        /* output MPEG4 video format */
    chConfig.endpointB.level   = TRC_MPEG4_LEVEL_0;         /* output MPEG4 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 40;                      /* input H.263 video at 40 kilobits per second */
    chConfig.endpointA.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    chConfig.endpointB.dataRate = 40;                       /* output MPEG4 video at 40 kilobits per second */
    chConfig.endpointB.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_2190;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_3016;

    /* ----- how transcoder inputs from and outputs to endpoint A ----- */

    /* activate receive of RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointA.chanIn.rtcpRxTimeout = 0;                /* max time (in msecs) that can pass without receiving RTP or RTCP before considering endpoint timed out (0 = no timeout) */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointA.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to H.263 endpoint address */
    chConfig.endpointA.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to H.263 endpoint port */
    chConfig.endpointA.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointA.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointA.chanOut.rtcpTransmitter = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointA.chanOut.rtcpTxTimeout = 0;               /* max time (in msecs) that can pass without receiving RTCP before considering endpoint timed out (0 = no timeout) */

    /* ----- how transcoder inputs from and outputs to endpoint B ----- */

    /* activate receive of RTCP from endpoint B */
    chConfig.endpointB.chanIn.rtcpReceiver = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointB.chanIn.rtcpRxTimeout = 0;                /* max time (in msecs) that can pass without receiving RTP or RTCP before considering endpoint timed out (0 = no timeout) */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to MPEG4 endpoint address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to MPEG4 endpoint port */
    chConfig.endpointB.chanOut.payloadId = 100;                 /* MPEG4 uses payload IDs from 96 to 127 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointB.chanOut.rtcpTxTimeout = 0;               /* max time (in msecs) that can pass without receiving RTCP before considering endpoint timed out (0 = no timeout) */

    /* ----- full-duplex direction from endpoint B to endpoint A [TRC_DIR_FDX2] ----- */

    /* configure the MPEG4 decoder that is transcoding from MPEG-4 endpoint B to H.263 endpoint A */
    chConfig.decoder[TRC_DIR_FDX2].optData = sampleMpeg4ConfigDCI;
    chConfig.decoder[TRC_DIR_FDX2].optSize = sizeof(sampleMpeg4ConfigDCI);

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to the H.263 endpoint ----- */
    result = sampleGenIframe( chHandle, TRC_DIR_FDX1 );  /* generate I-FRAME out duplex channel */
    if (result == TRC_SUCCESS)
    {
        /* ----- cause the transcoder to generate an I-FRAME to the MPEG4 endpoint ----- */
        result = sampleGenIframe( chHandle, TRC_DIR_FDX2 );  /* generate I-FRAME out duplex channel */
    }
    /* If an error occurs in either call, we are stopping and destroying the channel anyways... */


    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleFullDuplexM2H - create a full-duplex connection (btw MPEG4 and H.263)
 *
 * This example takes a full-duplex channel through a standard call flow.
 *
 * The example connects an MPEG4 endpoint (operating at 40 kbps and 7 fps)
 * with an H.263 endpoint (operating at 40 kbps and 7 fps).
 *
 *                      FDX1 ===> |          | ===> FDX1
 * (Endpoint A [MPEG4])           |TRANSCODER|           (Endpoint B [H.263])
 *                      FDX2 <=== |          | <=== FDX2
 *****************************************************************************/
void sampleFullDuplexM2H( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "FULL-DUPLEX channel: MPEG4 <=> H.263" );

    /* ----- create a full-duplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_FDX, "sampleFullDuplexM2H" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );       /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_MPEG4;         /* input MPEG4 video format */
    chConfig.endpointA.profile = TRC_PROFILE_SIMPLE;        /* input MPEG4 video format */
    chConfig.endpointA.level   = TRC_MPEG4_LEVEL_0;         /* input MPEG4 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_H263;          /* output H.263 video format */
    chConfig.endpointB.profile = TRC_PROFILE_BASELINE;      /* output H.263 video format */
    chConfig.endpointB.level   = TRC_H263_LEVEL_10;         /* output H.263 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 40;                      /* input H.263 video at 40 kilobits per second */
    chConfig.endpointA.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    chConfig.endpointB.dataRate = 40;                       /* output MPEG4 video at 40 kilobits per second */
    chConfig.endpointB.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_3016;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_2429;

    /* ----- how transcoder inputs from and outputs to endpoint A ----- */

    /* do not listen for RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointA.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to MPEG4 endpoint address */
    chConfig.endpointA.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to MPEG4 endpoint port */
    chConfig.endpointA.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointA.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointA.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

    /* ----- how transcoder inputs from and outputs to endpoint B ----- */

    /* do not listen for RTCP from endpoint B */
    chConfig.endpointB.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to H.263 endpoint address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to H.263 endpoint port */
    chConfig.endpointB.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

    /* ----- full-duplex direction from endpoint A to endpoint B [TRC_DIR_FDX1] ----- */

    /* configure the MPEG4 decoder that is transcoding from MPEG-4 endpoint B to H.263 endpoint A */
    chConfig.decoder[TRC_DIR_FDX1].optData = sampleMpeg4ConfigDCI;
    chConfig.decoder[TRC_DIR_FDX1].optSize = sizeof(sampleMpeg4ConfigDCI);

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to the H.263 endpoint ----- */
    result = sampleGenIframe( chHandle, TRC_DIR_FDX1 );  /* generate I-FRAME out duplex channel */
    if (result == TRC_SUCCESS)
    {
        /* ----- cause the transcoder to generate an I-FRAME to the MPEG4 endpoint ----- */
        result = sampleGenIframe( chHandle, TRC_DIR_FDX2 );  /* generate I-FRAME out duplex channel */
    }
    /* If an error occurs in either call, we are stopping and destroying the channel anyways... */


    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleFullDuplexH2H - create a full-duplex connection (both sides H.263)
 *
 * This example takes a full-duplex channel through a standard call flow.
 *
 * Even when both sides of a connection are using the same video encoding type
 * (H.263 in this example), transcoding is still required to convert from
 * one type of frame resolution / data rate / frame rate to another.
 * In this example, an H.263 endpoint using CIF is connected to an H.263
 * endpoint using QCIF.
 *
 *                      FDX1 ===> |          | ===> FDX1
 * (Endpoint A [CIF])             |TRANSCODER|           (Endpoint B [QCIF])
 *                      FDX2 <=== |          | <=== FDX2
 *****************************************************************************/
void sampleFullDuplexH2H( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "FULL-DUPLEX channel: H.263 <=> H.263" );

    /* ----- create a full-duplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_FDX, "sampleFullDuplexH2H" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );       /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_H263;          /* input H.263 video format */
    chConfig.endpointA.profile = TRC_PROFILE_BASELINE;      /* input H.263 video format */
    chConfig.endpointA.level   = TRC_H263_LEVEL_10;         /* input H.263 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_H263;          /* output H.263 video format */
    chConfig.endpointB.profile = TRC_PROFILE_BASELINE;      /* output H.263 video format */
    chConfig.endpointB.level   = TRC_H263_LEVEL_10;         /* output H.263 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 200;                     /* input H.263 video at 200 kilobits per second */
    chConfig.endpointA.frameRate = 15;                      /* 15 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_CIF;       /* CIF (352 x 288) frame resolution */

    chConfig.endpointB.dataRate = 40;                       /* output H.263 video at 40 kilobits per second */
    chConfig.endpointB.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_2429;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_2190;

    /* ----- how transcoder inputs from and outputs to endpoint A ----- */

    /* do not listen for RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointA.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to CIF endpoint address */
    chConfig.endpointA.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to CIF endpoint port */
    chConfig.endpointA.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointA.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointA.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

    /* ----- how transcoder inputs from and outputs to endpoint B ----- */

    /* do not listen for RTCP from endpoint B */
    chConfig.endpointB.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to QCIF endpoint address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to QCIF endpoint port */
    chConfig.endpointB.chanOut.payloadId = 34;                  /* H.263 uses payload ID 34 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to the H.263 endpoint ----- */
    result = sampleGenIframe( chHandle, TRC_DIR_FDX1 );  /* generate I-FRAME out duplex channel */
    if (result == TRC_SUCCESS)
    {
        /* ----- cause the transcoder to generate an I-FRAME to the MPEG4 endpoint ----- */
        result = sampleGenIframe( chHandle, TRC_DIR_FDX2 );  /* generate I-FRAME out duplex channel */
    }
    /* If an error occurs in either call, we are stopping and destroying the channel anyways... */


    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleFullDuplexM2M - create a full-duplex connection (both sides MPEG4)
 *
 * This example takes a full-duplex channel through a standard call flow.
 *
 * Even when both sides of a connection are using the same video encoding type
 * (MPEG4 in this example), transcoding is still required to convert from
 * one type of data rate / frame rate to another.
 * In this example, an MPEG4 endpoint operating at 30 kbps and 7 fps
 * is connected to an MPEG4 endpoint operating at 40 kbps and 10 fps.
 *
 *                       FDX1 ===> |          | ===> FDX1
 * (Endpoint A [30 @ 7])           |TRANSCODER|           (Endpoint B [40 @ 10])
 *                       FDX2 <=== |          | <=== FDX2
 *****************************************************************************/
void sampleFullDuplexM2M( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

    sampleBanner( "FULL-DUPLEX channel: MPEG4 <=> MPEG4" );

    /* ----- create a full-duplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_FDX, "sampleFullDuplexM2M" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );       /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_MPEG4;         /* input MPEG4 video format */
    chConfig.endpointA.profile = TRC_PROFILE_SIMPLE;        /* input MPEG4 video format */
    chConfig.endpointA.level = TRC_MPEG4_LEVEL_0;           /* input MPEG4 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_MPEG4;         /* input MPEG4 video format */
    chConfig.endpointB.profile = TRC_PROFILE_SIMPLE;        /* input MPEG4 video format */
    chConfig.endpointB.level = TRC_MPEG4_LEVEL_0;           /* input MPEG4 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 30;                      /* input MPEG4 video at 30 kilobits per second */
    chConfig.endpointA.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    chConfig.endpointB.dataRate = 40;                       /* output MPEG4 video at 40 kilobits per second */
    chConfig.endpointB.frameRate = 10;                      /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_3016;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_3016;

    /* ----- how transcoder inputs from and outputs to endpoint A ----- */

    /* do not listen for RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointA.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to endpoint A address */
    chConfig.endpointA.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to endpoint A port */
    chConfig.endpointA.chanOut.payloadId = 111;                 /* MPEG4 uses payload IDs from 96 to 127 */
    chConfig.endpointA.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointA.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */

    /* ----- how transcoder inputs from and outputs to endpoint B ----- */

    /* do not listen for RTCP from endpoint B */
    chConfig.endpointB.chanIn.rtcpReceiver = TRC_RTCP_DISABLED; /* do not listen for or send RTCP */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "127.0.0.1" );   /* !!! destination IP address should be set to endpoint B address */
    chConfig.endpointB.chanOut.rtpPort = 2020;                  /* !!! destination port should be set to endpoint B port */
    chConfig.endpointB.chanOut.payloadId = 122;                 /* MPEG4 uses payload IDs from 96 to 127 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_DISABLED;  /* do not listen for or send RTCP */


    /* ----- set MPEG4 optional decoder configuration for both directions ----- */

    /* configure the MPEG4 decoder that is transcoding from endpoint A to endpoint B */
    chConfig.decoder[TRC_DIR_FDX1].optData = sampleMpeg4ConfigDCI;
    chConfig.decoder[TRC_DIR_FDX1].optSize = sizeof(sampleMpeg4ConfigDCI);

    /* configure the MPEG4 decoder that is transcoding from endpoint B to endpoint A */
    chConfig.decoder[TRC_DIR_FDX2].optData = sampleMpeg4ConfigDCI;
    chConfig.decoder[TRC_DIR_FDX2].optSize = sizeof(sampleMpeg4ConfigDCI);

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* ----- cause the transcoder to generate an I-FRAME to the H.263 endpoint ----- */
    result = sampleGenIframe( chHandle, TRC_DIR_FDX1 );  /* generate I-FRAME out duplex channel */
    if (result == TRC_SUCCESS)
    {
        /* ----- cause the transcoder to generate an I-FRAME to the MPEG4 endpoint ----- */
        result = sampleGenIframe( chHandle, TRC_DIR_FDX2 );  /* generate I-FRAME out duplex channel */
    }
    /* If an error occurs in either call, we are stopping and destroying the channel anyways... */


    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleOverlaySpxH2M - create a simplex connection (H.263 to MPEG4 endpoints)
 *                       with text and image overlays.
 *
 * This example takes a simplex channel through a standard call flow.
 *
 * The example connects an H.263 endpoint to an MPEG4 endpoint outputting
 * at 40 kilobits/sec and 7 frames/sec.
 *
 * (Endpoint A [H.263])   ===> |TRANSCODER| ===>   (Endpoint B [MPEG4])
 *****************************************************************************/
void sampleOverlaySpxH2M( )
{
    U32             result;
    TRC_HANDLE      chHandle;
    tTrcChConfig    chConfig;

   sampleBanner( "SIMPLEX channel: H.263 ==> MPEG4 with Overlays" );

    /* ----- create a simplex channel ----- */
    result = sampleCreateChannel( &chHandle, TRC_CH_OVERLAY + TRC_CH_SIMPLEX, "sampleOverlaySpxH2M" );
    if (result != TRC_SUCCESS)
    {
        return;
    }

    /* ----- configure channel for connecting an H.263 endpoint to an MPEG4 endpoint */
    memset( &chConfig, 0, sizeof(tTrcChConfig) );           /* always zero-fill for future backward compatiblity */

    /* set encoding type and video frame resolution for each endpoint */
    chConfig.endpointA.vidType = TRC_VIDTYPE_H263;          /* input H.263 video format */
    chConfig.endpointA.profile = TRC_PROFILE_BASELINE;      /* input H.263 video format */
    chConfig.endpointA.level   = TRC_H263_LEVEL_10;         /* input H.263 video format */

    chConfig.endpointB.vidType = TRC_VIDTYPE_MPEG4;         /* output MPEG4 video format */
    chConfig.endpointB.profile = TRC_PROFILE_SIMPLE;        /* output MPEG4 video format */
    chConfig.endpointB.level   = TRC_MPEG4_LEVEL_0;         /* output MPEG4 video format */

    /* set the data rate, frame rate and frame resolution for each endpont */
    chConfig.endpointA.dataRate  = 40;                      /* input H.263 video at 40 kilobits per second */
    chConfig.endpointA.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointA.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    chConfig.endpointB.dataRate = 40;                       /* output MPEG4 video at 40 kilobits per second */
    chConfig.endpointB.frameRate = 7;                       /* 7 frames per second */
    chConfig.endpointB.frameRes  = TRC_FRAME_RES_QCIF;      /* QCIF (176 x 144) frame resolution */

    /* set the packetization mode in use by each endpoint */
    chConfig.endpointA.packetizeMode = TRC_PACKETIZE_2190;

    chConfig.endpointB.packetizeMode = TRC_PACKETIZE_3016;

    /* activate receive of RTCP from endpoint A */
    chConfig.endpointA.chanIn.rtcpReceiver = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointA.chanIn.rtcpRxTimeout = 0;                /* max time (in msecs) that can pass without receiving RTP or RTCP before considering endpoint timed out (0 = no timeout) */

    /* set dummy values for destination addressing */
    strcpy( chConfig.endpointB.chanOut.ipAddr, "10.10.20.4" );   /* !!! destination IP address should be set to MPEG4 endpoint address */
    chConfig.endpointB.chanOut.rtpPort = 6010;                  /* !!! destination port should be set to MPEG4 endpoint port */
    chConfig.endpointB.chanOut.payloadId = 100;                 /* MPEG4 uses payload IDs from 96 to 127 */
    chConfig.endpointB.chanOut.tos = 0;                         /* standard tos (use higher values to control quality of service) */
    chConfig.endpointB.chanOut.rtcpTransmitter = TRC_RTCP_ENABLED;  /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */
    chConfig.endpointB.chanOut.rtcpTxTimeout = 0;               /* max time (in msecs) that can pass without receiving RTCP before considering endpoint timed out (0 = no timeout) */

      /* ----- start the channel ----- */
    result = sampleStartChannel( chHandle, &chConfig );
    if (result != TRC_SUCCESS)
    {
        trcDestroyVideoChannel( chHandle );
        return;
    }

    /* Generate a series of overlays */
    result = sampleTestOverlays(chHandle, TRC_DIR_SIMPLEX);

    /* ----- stop the channel ----- */
    /* Normally, a channel is left in the started state for the duration of a
     * call between the two endpoints. When the controlling application detects
     * that the call has completed, the channel must be stopped and then destroyed,
     * returning the channel to the free pool for use in another call.
     */
    sampleStopChannel( chHandle );            /* stop the channel (waiting for STOP DONE) */

    /* ----- destroy the stopped channel, making channel available for use in another call ----- */
    sampleDestroyChannel( chHandle );
}

/*****************************************************************************
 * sampleEventHandler - handle event notifications from TRC API
 *
 * This function is called as an upcall from the an execution thread
 * of the TRC API. The event handler routine must pass these notifications
 * to the application in a thread-safe manner.
 *
 * inputs: pMsg - pointer to event message provided by TRC API
 *         size - byte length of message
 *
 * output: reserved for future use - always return 0
 *****************************************************************************/
U32 sampleEventHandler( tTrcMessage *pMsg, int size )
{
    int     rc;
    int     releaseSemaphore = 1;   /* default: release the sample progress control semaphore */
    S8 *eventName = trcValueName(TRCVALUE_EVENT, pMsg->event);
    S8 *resultName = trcValueName(TRCVALUE_RESULT, pMsg->result);

    if (debug)
    {
        printf("TRC Event [%s]: result [%s]\n",eventName, resultName);
    }

    switch( pMsg->event)
    {
        /* ----- unexpected channel messages (error indications) ----- */

        case TRCEVN_CHANNEL_LOST:
            printf("!!! Channel connection lost [userKey 0x%X]\n",pMsg->userKey);
            releaseSemaphore = 0;       /* unexpected event so do not release semaphore */
            break;

        case TRCEVN_CHANNEL_RECOVERED:
            printf("!!! Channel connection recovered [userKey 0x%X]\n",pMsg->userKey);
            releaseSemaphore = 0;       /* unexpected event so do not release semaphore */
            break;

        case TRCEVN_CHANNEL_FAILED:
            printf("!!! Channel connection failed [userKey 0x%X]\n",pMsg->userKey);
            releaseSemaphore = 0;       /* unexpected event so do not release semaphore */
            break;

        case TRCEVN_CHANNEL_RTCP_BYE:
            if (strlen((S8 *)&pMsg->data[TRCDATA_CHANNEL_RTCP_REASON]) == 0)
            {   /* RTCP BYE was received with no optional BYE reason provided */
                printf("!!! Channel received an RTCP BYE from endpoint %s [direction %s] [userKey 0x%X]\n",
                       trcValueName( TRCVALUE_ENDPOINT, pMsg->data[TRCDATA_CHANNEL_RTCP_ENDPOINT] ),
                       trcValueName( TRCVALUE_DIRECTION, pMsg->data[TRCDATA_CHANNEL_RTCP_DIRECTION] ),
                       pMsg->userKey);
            }
            else
            {   /* RTCP BYE was received with BYE reason provided */
                printf("!!! Channel received an RTCP BYE from endpoint %s [direction %s] [reason:%s] [userKey 0x%X]\n",
                       trcValueName( TRCVALUE_ENDPOINT, pMsg->data[TRCDATA_CHANNEL_RTCP_ENDPOINT] ),
                       trcValueName( TRCVALUE_DIRECTION, pMsg->data[TRCDATA_CHANNEL_RTCP_DIRECTION] ),
                       (S8 *)&pMsg->data[TRCDATA_CHANNEL_RTCP_REASON],
                       pMsg->userKey);
            }
            releaseSemaphore = 0;       /* unexpected event so do not release semaphore */
            break;

        case TRCEVN_CHANNEL_RTCP_TIMEOUT:
            printf("!!! Channel endpoint %s protocol %s timeout [direction %s] [userKey 0x%X]\n",
                   trcValueName( TRCVALUE_ENDPOINT, pMsg->data[TRCDATA_CHANNEL_RTCP_ENDPOINT] ),
                   trcValueName( TRCVALUE_PROTOCOL, pMsg->data[TRCDATA_CHANNEL_RTCP_PROTOCOL] ),
                   trcValueName( TRCVALUE_DIRECTION, pMsg->data[TRCDATA_CHANNEL_RTCP_DIRECTION] ),
                   pMsg->userKey);
            releaseSemaphore = 0;       /* unexpected event so do not release semaphore */
            break;

        case TRCEVN_RESOURCE_CHANGE:    /* indication that number of available transcoder resources has changed */
            printf("... Resource Change (previous %u, now %u licenses [O:%u(x:%u)] available)\n",
                   pMsg->data[TRCDATA_RESOURCE_CHANGE_PREVIOUS],
                   pMsg->data[TRCDATA_RESOURCE_CHANGE_AVAILABLE],
                   pMsg->data[TRCDATA_RESOURCE_OVL_LICENSE],
                   pMsg->data[TRCDATA_RESOURCE_OVL_EXCLUSIVE]);
            releaseSemaphore = 0;       /* disregard RESOURCE_CHANGE events (sample just sleeps until all VTPs are available) */
            break;

        case TRCEVN_CHANNEL_OVL_EVENT:    /* Overlay related event indocation */
            switch (pMsg->result)
            {
                case TRC_OVLEVT_TRCR_RENDER_SUCCESS     :
                case TRC_OVLEVT_VTC_SCROLL_END          :
                    /* Informational, silently discard. */
                    break;

                default:
                    printf("!!! Overlay error indication received on channel [ch userkey:0x%X] (ovl userkey:0x%X] [%s] [res=0x%X]\n",
                           pMsg->userKey,
                           pMsg->data[TRCDATA_OVERLAY_USERKEY],
                           trcValueName( TRCVALUE_RESULT, pMsg->result),
                           pMsg->result);
                    break;
            }
            break;

        /* ----- asynchronous events that sample waits for ----- */
        /* NOTE: trace these notifications after switching to the application thread */

        case TRCEVN_CREATE_CHANNEL_DONE:    /* completion of a call to trcCreateVideoChannel */
        case TRCEVN_START_CHANNEL_DONE:     /* completion of a call to trcStartVideoChannel */
        case TRCEVN_STOP_CHANNEL_DONE:      /* completion of a call to trcStopVideoChannel */
        case TRCEVN_IFRAME_CHANNEL_DONE:    /* completion of a call to trcIframeVideoChannel */
        case TRCEVN_DESTROY_CHANNEL_DONE:   /* completion of a call to trcDestroyVideoChannel */
        case TRCEVN_SHUTDOWN_DONE:          /* completion of a call to trcShutdown */
        case TRCEVN_CREATE_OVL_DONE:           /* completion of a call to trcCreateOverlay */
        case TRCEVN_DESTROY_OVL_DONE:          /* completion of a call to trcDestroyOverlay */
        case TRCEVN_START_OVL_DONE:            /* completion of a call to trcStartOverlay */
        case TRCEVN_STOP_OVL_DONE:             /* completion of a call to trcStopOverlay */
        case TRCEVN_CHANGE_OVL_CFG_DONE:       /* completion of a call to trcChangeOverlayConfig */
        case TRCEVN_CHANGE_OVL_CONTENT_DONE:   /* completion of a call to trcChangeOverlayContent */
        case TRCEVN_CHANGE_OVL_SCROLLING_DONE: /* completion of a call to trcChangeOverlayScrolling */
            break;

        /* ----- unknown event handler ----- */

        default:
            printf( "event %d (0x%X) is not a valid TRC event!\n",
                    pMsg->event, pMsg->event );
            releaseSemaphore = 0;       /* unexpected event so do not release semaphore */

    }   /* end switch */

    if (releaseSemaphore)
    {
        /* sample wakeup message - use semaphore mechanism to safely switch control to application */
        memcpy( &lastMsg, pMsg, size );
        rc = sampleReleaseSemaphore( &sampleSemaphore );
        if (rc != 0)
        {
            printf( "!!! ERROR: %d releasing semaphore\n", rc );
        }
    }

    return( 0 );        /* always return 0 - reserved for future use */
}

/*###########################################################################*
 * MAIN
 *###########################################################################*/
int main(int argc, char *argv[])
{
    char           *myname;
#ifdef UNIX
    char           *config_file = "/opt/nms/trc/trcapi.cfg";
#else /* UNIX */
    char           *config_file = "c:\\nms\\trc\\trcapi.cfg";
#endif /* UNIX */
    U32             detailVtpId = 0;    /* which VTP to show detailed status for (0 = NOT showing VTP detailed status) */
    unsigned char   detail_status = 0;  /* default: show summary status */
    unsigned char   reset_mode = 0;     /* default: not executing reset example */
    int             reset_vtp;          /* which VTP to reset */
    int             arg_index = 1;      /* index to next parameter to parse */
    U32             result = TRC_SUCCESS;   /* no error encountered yet */
    int             rc;
    tTrcUsage       trcUse;
    int             value;
    int             i;
    U32             enableCnt;

    /* ----- process command-line arguments ----- */
    myname = argv[0];
    printf( "%s - TRC sample application\n", myname );

    /* ----- initialize the API ----- */
    /* Set tracing before calling any other API function.
     * This allows even trcInitialize to trace it's flow if necessary.
     * When tracing trcInitialize, add stdout tracing: trcSetTrace( TRCTR_ERR, TRCTR_ERR ).
     * This allows tracing before the log file is created.
     */
    if (debug)
    {
        trcSetTrace( -1, -1 );                  /* log everything (to console and to file) */
    }
    else
    {
        trcSetTrace( 0, TRCTR_ERR|TRCTR_ALM );  /* no logging to stdout; log any error or alarm (warning) encountered to the log file */
    }

    while (arg_index < argc)
    {
        if ( (argv[arg_index][0] != '-') || (strlen(argv[arg_index]) < 2) )
        {
            printf( "!!! ERROR: Expecting parameters in form: -<option> <value> where options include: -c, -s, -r\n" );
            exit(1);
        }

        switch( argv[arg_index][1] )
        {
            case 'c':       /* configuration file */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -c <apiConfigFile>\n" );
                    exit(1);
                }

                config_file = argv[arg_index + 1];
                arg_index += 2;
                break;

            case 's':       /* show detailed VTP status */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -s <VTPtoShow>\n" );
                    exit(1);
                }

                if (sscanf( argv[arg_index + 1], "%d", &detailVtpId ) != 1)
                {
                    printf( "!!! ERROR: Expecting -s <VTPtoShow> where <VTPtoShow> is VTP number.\n" );
                    exit(1);
                }

                if (detailVtpId == 0)
                {
                    printf( "!!! ERROR: Expecting -s <VTPtoShow> where <VTPtoShow> is 1-based VTP number.\n" );
                    exit(1);
                }

                detail_status = 1;
                arg_index += 2;
                break;

            case 'r':       /* reset a VTP */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -r <VTPtoReset>\n" );
                    exit(1);
                }

                if (sscanf( argv[arg_index + 1], "%d", &reset_vtp ) != 1)
                {
                    printf( "!!! ERROR: Expecting -r <VTPtoReset> where <VTPtoReset> is VTP number.\n" );
                    exit(1);
                }

                if (reset_vtp == 0)
                {
                    printf( "!!! ERROR: Expecting -r <VTPtoReset> where <VTPtoReset> is 1-based VTP number.\n" );
                    exit(1);
                }

                reset_mode = 1;
                arg_index += 2;
                break;

            case 't':       /* set full tracing */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -t <trace value>\n" );
                    exit(1);
                }

                if (sscanf( argv[arg_index + 1], "%d", &value ) != 1)
                {
                    printf( "!!! ERROR: Expecting -t <trace value>\n" );
                    exit(1);
                }
                trcSetTrace( value, value );
                arg_index += 2;
                break;

            case '?':
            case 'h':
                printf( "Command line options: -c <config file> -s <show VTP> -r <reset VTP>\n" );
                exit(0);
                break;

            default:
                printf( "!!! ERROR: Unknown option [%c] -  expecting: -c <config file> -s <show VTP> -r <reset VTP>\n",
                        argv[arg_index][1] );
                exit(1);
        }
    }

    /* create a semaphore to be used for synchronization with event handler */
    rc = sampleCreateSemaphore( &sampleSemaphore );
    if (rc != 0)
    {
        printf( "!!! ERROR: %d creating semaphore\n", rc );
        trcShutdown( );
        exit(1);
    }

    /* Initialize the TRC module.
     * Pass configuration file, log file, and notification upcall to the TRC API;
     * activating connection with each VTP listed in the configuration file.
     */
    result = trcInitialize( TRC_CTL_VERSION, TRC_CTL_REVISION, "trc_sample", config_file, "trcsample.log", sampleEventHandler );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcInitialize.\n",
                trcValueName(TRCVALUE_RESULT, result) );
        exit(1);
    }

    /* Immediately request information to determine the number of VTPs defined */
    result = trcVTPStatus ( &currTrcStatus );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcVTPStatus.\n",
                trcValueName(TRCVALUE_RESULT, result) );
        exit(1);
    }
    if (currTrcStatus.vtpDefined == 0)
    {
        printf( "!!! ERROR: No VTPs defined in configuration file %s.\n",
                config_file );
        exit(1);
    }
    if (detailVtpId > currTrcStatus.vtpDefined)
    {
        printf( "!!! ERROR: Only %d VTP(s) defined\n",
                currTrcStatus.vtpDefined );
        exit(1);
    }

    /* wait for all defined VTPs to become available */
    /* NOTE: An application need only wait for a single TRCEVN_RESOURCE_CHANGE event before creating channels.
     *       As a simplification, the sample application just sleeps until all VTPs listed in the TRC API
     *       configuration file have become available (enabled).
     */
    do
    {
        printf( "... waiting for all VTP connections to become enabled ...\n" );
        SLEEP(1);       /* sleep for 1 second to allow time for all VTP connections to be established */
        sleepCnt++;

        /* Obtain updated information regarding all VTP connections */
        result = trcVTPStatus( &currTrcStatus );
        if (result != TRC_SUCCESS)
        {
            printf( "!!! ERROR: %s returned from trcVTPStatus.\n",
                    trcValueName(TRCVALUE_RESULT, result) );
            exit(1);
        }

        enableCnt = 0;
        for (i = 0; i < currTrcStatus.vtpDefined; i++)
        {
            if (currTrcStatus.vtp[i].state == TRCVTP_STATE_ENABLED)
            {
                enableCnt++;
            }
        }

    } while ( (enableCnt != currTrcStatus.vtpDefined) && (sleepCnt <= 5) );

    if (enableCnt != currTrcStatus.vtpDefined)
    {   /* we've slept for more than 5 seconds and all VTPs have not come up */
        for (i = 0; i < currTrcStatus.vtpDefined; i++)
        {
            if (currTrcStatus.vtp[i].state != TRCVTP_STATE_ENABLED)
            {
                printf( "!!! ERROR: VTP %d not enabled (state = %s)\n",
                        i + 1, trcValueName( TRCVALUE_VTPSTATE, currTrcStatus.vtp[i].state ) );
            }
        }
        exit(1);
    }

    /* ----- show detailed VTP status if requested ----- */
    if (detail_status)
    {   /* display detailed status information for the specified VTP */
        sampleDetailStatus( detailVtpId );
    }

    /* ----- execute the sample code indicated by command line ----- */
    else if (reset_mode)
    {   /* reset the specified VTP */
        printf( "Resetting VTP %d...\n", reset_vtp );
        result = trcResetVTP( reset_vtp );
        if (result != TRC_SUCCESS)
        {
            printf( "!!! ERROR: %s returned from trcResetVTP\n",
                    trcValueName(TRCVALUE_RESULT, result) );
        }
    }

    else
    {   /* create,start,config,stop,destroy a series of example channels */

        /* perform each channel usage example */
        sampleSimplexH2M( );    /* SIMPLEX connection from H.263 to MPEG4 */

        sampleSimplexM2H( );    /* SIMPLEX connection from MPEG4 to H.263 */

        sampleSimplexH2H( );    /* SIMPLEX connection between two H.263 endpoints */

        sampleSimplexM2M( );    /* SIMPLEX connection between two MPEG4 endpoints */

        sampleFullDuplexH2M( ); /* FULL-DUPLEX connection between H.263 and MPEG4 endpoints */

        sampleFullDuplexM2H( ); /* FULL-DUPLEX connection between MPEG4 and H.263 endpoints */

        sampleFullDuplexH2H( ); /* FULL-DUPLEX connection between two H.263 endpoints */

        sampleFullDuplexM2M( ); /* FULL-DUPLEX connection between two MPEG4 endpoints */

        sampleOverlaySpxH2M( ); /* SIMPLEX connection from H.263 to MPEG4 with overlays*/

        /* show channel usage information */
        result = trcUsage( &trcUse );
        if (result == TRC_SUCCESS)
        {
            printf( "RESOURCE USAGE: %d licenses available on %d VTP(s)\n",
                    trcUse.licensesAvail, trcUse.vtpDefined );
            printf( "                %d VTP(s) currently available\n",
                    trcUse.vtpAvail );
            printf( "                Local SIMPLEX channels: %03d, Local FDX channels: %03d\n",
                    trcUse.simplexLocal, trcUse.fdxLocal );
            printf( "                *ALL* SIMPLEX channels: %03d, *ALL* FDX channels: %03d\n",
                    trcUse.simplexTotal, trcUse.fdxTotal );
        }
        else
        {
            printf( "!!! ERROR: %s returned from trcUsage.\n",
                    trcValueName(TRCVALUE_RESULT, result) );
        }
    }

    /* ----- shutdown the API ----- */
    result = trcShutdown( );
    if (result != TRC_SUCCESS)
    {
        printf( "!!! ERROR: %s returned from trcShutdown.\n",
                trcValueName( TRCVALUE_RESULT, result ) );
        exit(1);
    }
    if (debug)
    {
        printf( "    TRC SHUTDOWN... in progress (awaiting SHUTDOWN DONE event)\n" );
    }
    rc = sampleWaitSemaphore( &sampleSemaphore );
    if ( rc != 0)
    {
        printf( "!!! ERROR: %d waiting on semaphore\n", rc );
        exit(1);
    }
    else
    {
        if ( (lastMsg.event == TRCEVN_SHUTDOWN_DONE) &&
             (lastMsg.result == TRC_SUCCESS) )
        {
            if (debug)
            {
                printf( "    ...TRC SHUTDOWN completed successfully\n" );
            }
        }
        else
        {
            printf( "!!! ERROR: Unexpected event [%s] or result [%s] awaiting TRC SHUTDOWN\n",
                    trcValueName(TRCVALUE_EVENT, lastMsg.event),
                    trcValueName(TRCVALUE_RESULT, lastMsg.result) );
        }
    }

    /* destroy the semaphore */
    rc = sampleDestroySemaphore( &sampleSemaphore );
    if (rc != 0)
    {
        printf( "!!! ERROR: %d while destroying semaphore\n", rc );
        exit(1);
    }

    return (0);
}
