/*****************************************************************************
 * vtmgr.c - Video Transcoder Manager Sample Application (using VTMNG API)
 *
 * This sample application shows how to use the VTMNG API to manage a set of
 * Video Transcoder Platforms (VTPs). VTPs are external servers which provide
 * the resources needed for video transcoding.
 *
 * External control applications connect to VTPs. Each application connection
 * instance is referred to an an APP. Applications create transcoder channels
 * (or CHNs). The VTP-based processes used to provide transcoder resources are
 * monitored (as MONs). Processes owned by the video transcoder process
 * monitor are automatically recovered in the event of abnormal termination.
 *****************************************************************************/

/*****************************************************************************
 **
 **                         PROPRIETARY NOTICE
 **
 **                       Copyright (c) 2000-2007
 **                 By NMS Communications Corporation.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of NMS Communications Corporation.
 **
 *****************************************************************************/

/*****************************************************************************
 * Sample Application Overview:
 *
 * Working examples of all normally used VTMNG API functions are provided:
 *
 * ===========================================================================
 * VTMNG API Function   Description
 * -------------------- ------------------------------------------------------
 * EXECUTION CONTROL
 *   vtMngInit          initialize and activate management communication interface
 *   vtMngPollLoop      enter polling loop, upcall on receives (management or keyboard)
 *   vtMngShutdown      shutdown all usage of the Video Transcoder Management interface
 *
 * MANAGEMENT REQUESTS
 *   vtMngGetAppList    Issue a request for the list of connected applications
 *   vtMngGetMonList    Issue a request for the list of processes being monitored
 *   vtMngGetChnList    Issue a request for the list of defined channels
 *   vtMngGetVtp        Issue a request for all VTP-level information
 *   vtMngGetApp        Issue a request for details of a particular application
 *   vtMngGetMon        Issue a request for details of a particular monitored process
 *   vtMngGetChn        Issue a request for details of a particular channel
 *   vtMngGetStTotal    Issue a request for the total (overall) statistics
 *   vtMngGetStCurrMin  Issue a request for the current minute's statistics
 *   vtMngGetHistPerMin Issue a request for a historical statistics record (per-minute)
 *   vtMngGetHistPerHHr Issue a request for a historical statistics record (per-half-hour)
 *   vtMngSetVtp        Issue a request to modify the VTP-level configuration
 *   vtMngEventVtp      Issue a request to perform a VTP-level event
 *   vtMngSetApp        Issue a request to modify a particular application's configuration
 *   vtMngEventApp      Issue a request to perform an application-level event
 *   vtMngSetMon        Issue a request to modify a particular monitored process's configuration
 *   vtMngEventMon      Issue a request to perform an event on a monitored process
 *   vtMngSetChn        Issue a request to modify a particular channel's configuration
 *   vtMngEventChn      Issue a request to perform a channel-level event
 *   vtMngZeroVtp       Issue a request to get and then zero the VTP-level statistics
 *   vtMngZeroApp       Issue a request to get and then zero an application's statistics
 *   vtMngZeroMon       Issue a request to get and then zero a monitored process's statistics
 *   vtMngZeroChn       Issue a request to get and then zero a channel's statistics
 *   vtMngZeroTotal     Issue a request to get and then zero the total statistics
 *
 * ERROR HANDLING:
 *   vtMngValueName     Display the ASCII name associated with a result code
 *
 * ===========================================================================
 * Command Line Syntax: vtmgr [-v <vtpAddr>]
 *                            [-f <eventLog>]
 *                            [-r <reqPort>]
 *                            [-t <trapPort>]
 *                            [-c <chnTrcLvl>]
 *                            [-d <dbgMask>]
 *                            [-R] [-T] [-K]
 *
 *   -v <vtpAddr>   specify a VTP address
 *   -f <eventLog>  set filename to log asynchronous events to [and reg for traps]
 *   -r <reqPort>   set the port number used to issue requests and receive responses
 *   -t <trapPort>  set the port number used to register for asynchronous traps
 *   -d <dbgMask>   specify debug log mask (use -d ? for more details)
 *   -c <chnTrcLvl> specify level of information to be traced on channel notifications
 *   -R             register to handle all received VT management responses
 *   -T             register to handle all received TRAPs (async notifications)
 *   -K             register to handle operator keyboard commands
 *
 * When -R, -T and -K are not specified:
 *
 *   vtmgr allows the VTMNG API to handle everything internally:
 *
 *     The VTMNG API presents the following operator command sets:
 *       [vtmng] is used for sending all the management requests
 *       [vtp] is used to control the VTP-level configuration
 *       [mon] is used to control the monitored-process-level configuration
 *
 *     Any received management response or TRAP is displayed to standard out.
 *
 * When -R, -T and -K are specified:
 *
 *   vtmgr acts as sample for response and TRAP receive handling.
 *   This source code provides a working example that can be used
 *   as a starting point for a custom manager program.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "vtmng.h"

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION    1             /* major or non-backwards compatible changes   */
#define MH_REVISION   3             /* minor fixes and/or enhancements             */

typedef struct __my_vtmng_info
{
    U32             id;
#define MY_VTMNG_ID 0x01020304

    VTMNG_VTPADDR   dest;           /* destination addressing information */
    U32             uniqueId;       /* unique identifier (used for certain commands) */
    VTMNG_VTP_CFG   vtpCfg;         /* area to hold changes to VTP-level config */
    VTMNG_VTP_CFG   vtpMask;        /* mask ID'ing which VTP-level fields are being changed */
    VTMNG_APP_CFG   appCfg;         /* area to hold changes to application-level config */
    VTMNG_APP_CFG   appMask;        /* mask ID'ing which application-level fields are being changed */
    VTMNG_MON_CFG   monCfg;         /* area to hold changes to monitored process-level config */
    VTMNG_MON_CFG   monMask;        /* mask ID'ing which monitored process-level fields are being changed */
    VTMNG_CHN_CFG   chnCfg;         /* area to hold changes to channel-level config */
    VTMNG_CHN_CFG   chnMask;        /* mask ID'ing which channel-level fields are being changed */

} MY_VTMNG_INFO;

/* ----- set log types set by default (see vtmng.h for descriptions) ----- */
#define MY_VTMNG_DEFAULT_LOG_MASK   (VTMNG_LOG_E2C|VTMNG_LOG_MON|VTMNG_LOG_CHN|VTMNG_LOG_APP|VTMNG_LOG_VTP|VTMNG_LOG_WRN|VTMNG_LOG_ERR)

/*###########################################################################*
 * GLOBALS
 *###########################################################################*/

MY_VTMNG_INFO   myInfo;     /* used as userkey to vtMngInit (provided as userkey to all upcalls) */

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 * PROTOTYPES
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*###########################################################################*
 * EXAMPLES RECEIVE HANDLER FUNCTIONS
 *###########################################################################*/

/*****************************************************************************
 * vtMgrGetAppListRsp - handle rx of response to GET APP LIST request
 *****************************************************************************/
void vtMgrGetAppListRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                         VTMNG_ENT_ID     *app,
                         U32               appCount )
{
    U32                 i;
    VTMNG_ENT_ID       *entId;

    if (result == VS_SUCCESS)
    {
        if (appCount == 0)
        {
            printf( "No applications currently connected.\n" );
        }
        else if (appCount == 1)
        {
            printf( "1 application currently connected:\n" );
        }
        else
        {
            printf( "%u applications currently connected:\n", appCount );
        }

        entId = &msg->rsp.getList.gListEnt[0];
        for (i = 0; i < appCount; i++)
        {
            if (entId->entName[0] != '\0')
            {
                printf( "  APP[%03.3u] ID = 0x%08X, NAME=%s\n",
                        i, entId->entObj.entUnique, entId->entName );
            }
            else
            {
                printf( "  APP[%03.3u] ID = 0x%08X\n",
                        i, entId->entObj.entUnique );
            }

            entId++;
        }
    }
    else
    {
        printf( "Error %s obtaining list of application connections\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
}

/*****************************************************************************
 * vtMgrGetMonListRsp - handle rx of response to GET MON LIST request
 *****************************************************************************/
void vtMgrGetMonListRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                         VTMNG_ENT_ID     *mon,
                         U32               monCount )
{
    U32                 i;
    VTMNG_ENT_ID       *entId;

    if (result == VS_SUCCESS)
    {
        if (monCount == 0)
        {
            printf( "No processes currently being monitored.\n" );
        }
        else if (monCount == 1)
        {
            printf( "1 process currently being monitored:\n" );
        }
        else
        {
            printf( "%u processes currently being monitored:\n", monCount );
        }

        entId = &msg->rsp.getList.gListEnt[0];
        for (i = 0; i < monCount; i++)
        {
            if (entId->entName[0] != '\0')
            {
                printf( "  MON[%03.3u] ID = 0x%08X, NAME=%s\n",
                        i, entId->entObj.entUnique, entId->entName );
            }
            else
            {
                printf( "  MON[%03.3u] ID = 0x%08X\n",
                        i, entId->entObj.entUnique );
            }

            entId++;
        }
    }
    else
    {
        printf( "Error %s obtaining list of processes being monitored\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
}

/*****************************************************************************
 * vtMgrGetChnListRsp - handle rx of response to GET CHN LIST request
 *****************************************************************************/
void vtMgrGetChnListRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                         VTMNG_ENT_ID     *chn,
                         U32               chnCount )
{
    U32                 i;
    VTMNG_ENT_ID       *entId;

    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining list of channels\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        if (chnCount == 0)
        {
            printf( "No channels currently defined.\n" );
        }
        else if (chnCount == 1)
        {
            printf( "1 channel currently defined:\n" );
        }
        else
        {
            printf( "%u channels currently defined:\n", chnCount );
        }

        entId = &msg->rsp.getList.gListEnt[0];
        for (i = 0; i < chnCount; i++)
        {
            if (entId->entName[0] != '\0')
            {
                printf( "  channel[%03.3u] ID = 0x%08X, NAME=%s\n",
                        i, entId->entObj.entUnique, entId->entName );
            }
            else
            {
                printf( "  channel[%03.3u] ID = 0x%08X\n",
                        i, entId->entObj.entUnique );
            }

            entId++;
        }
    }
}

/*****************************************************************************
 * vtMgrGetVtpRsp - handle rx of response to GET VTP request
 *****************************************************************************/
void vtMgrGetVtpRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     VTMNG_VTP_CFG        *cfg,
                     VTMNG_VTP_STATUS     *status,
                     VTMNG_VTP_STATS      *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining VTP information\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "VTP information obtained successfully\n" );
    }
}

/*****************************************************************************
 * vtMgrGetAppRsp - handle rx of response to GET APP request
 *****************************************************************************/
void vtMgrGetAppRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     U32                   appUnique,
                     VTMNG_APP_CFG        *cfg,
                     VTMNG_APP_STATUS     *status,
                     VTMNG_APP_STATS      *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining application information\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "Application [ID 0x%08X] information obtained successfully\n", appUnique );
    }
}

/*****************************************************************************
 * vtMgrGetMonRsp - handle rx of response to GET MON request
 *****************************************************************************/
void vtMgrGetMonRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     U32                   monUnique,
                     VTMNG_MON_CFG        *cfg,
                     VTMNG_MON_STATUS     *status,
                     VTMNG_MON_STATS      *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining monitored process information\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "Monitored Process [ID 0x%08X] information obtained successfully\n", monUnique );
    }
}

/*****************************************************************************
 * vtMgrGetChnRsp - handle rx of response to GET CHN request
 *****************************************************************************/
void vtMgrGetChnRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     U32                   chnUnique,
                     VTMNG_CHN_CFG        *cfg,
                     VTMNG_CHN_STATUS     *status,
                     VTMNG_CHN_STATS      *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining channel information (chnUnique=0x%08X)\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ),
                chnUnique );
    }
    else
    {
        printf( "Channel [ID 0x%08X] information obtained successfully\n", chnUnique );
    }
}

/*****************************************************************************
 * vtMgrGetStTotalRsp - handle rx of response to GET STATS TOTAL request
 *****************************************************************************/
void vtMgrGetStTotalRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                         VTMNG_ST_ENTRY   *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining TOTAL statistics\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "TOTAL statistics obtained successfully\n" );
    }
}

/*****************************************************************************
 * vtMgrGetStCurrMinRsp - handle rx of response to GET STATS CURRMIN request
 *****************************************************************************/
void vtMgrGetStCurrMinRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                           VTMNG_ST_ENTRY   *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining CURRENT MINUTE statistics\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "CURRENT MINUTE statistics obtained successfully\n" );
    }
}

/*****************************************************************************
 * vtMgrGetHistPerMinRsp - handle rx of response to GET HIST PERMIN request
 *****************************************************************************/
void vtMgrGetHistPerMinRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                            U32                minutesAgo,
                            VTMNG_ST_ENTRY    *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining PER-MINUTE history\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "PER-MINUTE statistics obtained successfully (minutesAgo=%u)\n", minutesAgo );
    }
}

/*****************************************************************************
 * vtMgrGetHistPerHHrRsp - handle rx of response to GET HIST PERHHR request
 *****************************************************************************/
void vtMgrGetHistPerHHrRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                            U32                halfHoursAgo,
                            VTMNG_ST_ENTRY    *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s obtaining PER-HALF-HOUR history\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "PER-HALF-HOUR statistics obtained successfully (halfHoursAgo=%u):\n", halfHoursAgo );
    }
}

/*****************************************************************************
 * vtMgrSetVtpRsp - handle rx of response to SET VTP request
 *****************************************************************************/
void vtMgrSetVtpRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     VTMNG_VTP_CFG            *cfg )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s setting VTP configuration\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "VTP configuration set successfully\n" );
    }
}

/*****************************************************************************
 * vtMgrEventVtpRsp - handle rx of response to EVENT VTP request
 *****************************************************************************/
void vtMgrEventVtpRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                       U32                       vtpEvent,
                       VTMNG_VTP_CFG            *cfg )
{
    switch (vtpEvent)
    {
        case VTMNG_VTP_E_RESTART:
            if (result != VS_SUCCESS)
            {
                printf( "Error %s eventing VTP RESTART\n",
                        vtMngValueName( VTMNG_VALUE_RESULT, result ) );
            }
            else
            {
                printf( "VTP RESTART success\n" );
            }
            break;

        case VTMNG_VTP_E_REBOOT:
            if (result != VS_SUCCESS)
            {
                printf( "Error %s eventing VTP REBOOT\n",
                        vtMngValueName( VTMNG_VALUE_RESULT, result ) );
            }
            else
            {
                printf( "VTP REBOOT success\n" );
            }
            break;

        default:
            printf( "Error: unexpected vtpEvent [0x%08X] on event VTP response\n", vtpEvent );
    }
}

/*****************************************************************************
 * vtMgrSetAppRsp - handle rx of response to SET APP request
 *****************************************************************************/
void vtMgrSetAppRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     U32                       appUnique,
                     VTMNG_APP_CFG            *cfg )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s setting APP configuration (appUnique=0x%08X)\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ),
                appUnique );
    }
    else
    {
        printf( "APP configuration set successfully (appUnique=0x%08X)\n", appUnique );
    }
}

/*****************************************************************************
 * vtMgrEventAppRsp - handle rx of response to EVENT APP request
 *****************************************************************************/
void vtMgrEventAppRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                       U32                       appUnique,
                       U32                       appEvent,
                       VTMNG_APP_CFG            *cfg )
{
    /* NOTE: There are no application-level events currently defined */
    printf( "Error: unexpected appEvent [0x%08X] on event APP response (appUnique=0x%08X)\n", appEvent, appUnique );
}

/*****************************************************************************
 * vtMgrSetMonRsp - handle rx of response to SET MON request
 *****************************************************************************/
void vtMgrSetMonRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     U32                       monUnique,
                     VTMNG_MON_CFG            *cfg )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s setting MON configuration (monUnique=0x%08X)\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ),
                monUnique );
    }
    else
    {
        printf( "MON configuration set successfully (monUnique=0x%08X)\n", monUnique );
    }
}

/*****************************************************************************
 * vtMgrEventMonRsp - handle rx of response to EVENT MON request
 *****************************************************************************/
void vtMgrEventMonRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                       U32                       monUnique,
                       U32                       monEvent,
                       VTMNG_MON_CFG            *cfg )
{
    switch (vtpAddr->sendkey)
    {
        case VTMNG_MON_E_TERMINATE:
            if (result != VS_SUCCESS)
            {
                printf( "Error %s eventing MON TERMINATE (monUnique=0x%08X)\n",
                        vtMngValueName( VTMNG_VALUE_RESULT, result ),
                        monUnique );
            }
            else
            {
                printf( "MON TERMINATE success (monUnique=0x%08X)\n", monUnique );
            }
            break;

        case VTMNG_MON_E_STOP:
            if (result != VS_SUCCESS)
            {
                printf( "Error %s eventing MON STOP (monUnique=0x%08X)\n",
                        vtMngValueName( VTMNG_VALUE_RESULT, result ),
                        monUnique );
            }
            else
            {
                printf( "MON STOP success (monUnique=0x%08X)\n", monUnique );
            }
            break;

        case VTMNG_MON_E_START:
            if (result != VS_SUCCESS)
            {
                printf( "Error %s eventing MON START (monUnique=0x%08X)\n",
                        vtMngValueName( VTMNG_VALUE_RESULT, result ),
                        monUnique );
            }
            else
            {
                printf( "MON START success (monUnique=0x%08X)\n", monUnique );
            }
            break;

        default:
            printf( "Error: unexpected monEvent [0x%08X] on event MON response (monUnique=0x%08X)\n", monEvent, monUnique );
    }
}

/*****************************************************************************
 * vtMgrSetChnRsp - handle rx of response to SET CHN request
 *****************************************************************************/
void vtMgrSetChnRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                     U32                       chnUnique,
                     VTMNG_CHN_CFG            *cfg )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s setting CHN configuration (chnUnique=0x%08X)\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ),
                chnUnique );
    }
    else
    {
        printf( "CHN configuration set successfully (chnUnique=0x%08X)\n", chnUnique );
    }
}

/*****************************************************************************
 * vtMgrEventChnRsp - handle rx of response to SET CHN request
 *****************************************************************************/
void vtMgrEventChnRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                       U32                       chnUnique,
                       U32                       chnEvent,
                       VTMNG_CHN_CFG            *cfg )
{
    /* NOTE: vtpAddr->sendkey was set when request was issued.
     *       This sample uses sendkey to easily recall whether set was due to an event.
     *       Other applications are free to use sendkey for other purposes.
     */
    switch (vtpAddr->sendkey)
    {
        case 0:                     /* response to a configuration request */
            if (result != VS_SUCCESS)
            {
                printf( "Error %s setting CHN configuration (chnUnique=0x%08X)\n",
                        vtMngValueName( VTMNG_VALUE_RESULT, result ),
                        chnUnique );
            }
            else
            {
                printf( "CHN configuration set successfully (chnUnique=0x%08X)\n", chnUnique );
            }
            break;

        case VTMNG_CHN_E_ABORT:
            if (result != VS_SUCCESS)
            {
                printf( "Error %s eventing CHN ABORT (chnUnique=0x%08X)\n",
                        vtMngValueName( VTMNG_VALUE_RESULT, result ),
                        chnUnique );
            }
            else
            {
                printf( "CHN ABORT success (chnUnique=0x%08X)\n", chnUnique );
            }
            break;

        default:
            printf( "Error: unexpected sendkey [0x%08X] on set CHN response (chnUnique=0x%08X)\n", vtpAddr->sendkey, chnUnique );
    }
}

/*****************************************************************************
 * vtMgrZeroVtpRsp - handle rx of response to ZERO VTP request
 *****************************************************************************/
void vtMgrZeroVtpRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                      VTMNG_VTP_STATS         *vtpStats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s zeroing VTP statistics\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "VTP statistics zeroed successfully\n" );
    }
}

/*****************************************************************************
 * vtMgrZeroAppRsp - handle rx of response to ZERO APP request
 *****************************************************************************/
void vtMgrZeroAppRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                      U32                      appUnique,
                      VTMNG_APP_STATS         *appStats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s zeroing APP statistics (appUnique=0x%08X)\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ),
                appUnique );
    }
    else
    {
        printf( "APP statistics zeroed successfully (appUnique=0x%08X):\n", appUnique );
    }
}

/*****************************************************************************
 * vtMgrZeroMonRsp - handle rx of response to ZERO MON request
 *****************************************************************************/
void vtMgrZeroMonRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                      U32                      monUnique,
                      VTMNG_MON_STATS         *monStats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s zeroing MON statistics (monUnique=0x%08X)\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ),
                monUnique );
    }
    else
    {
        printf( "MON statistics zeroed successfully (monUnique=0x%08X):\n", monUnique );
    }
}

/*****************************************************************************
 * vtMgrZeroChnRsp - handle rx of response to ZERO CHN request
 *****************************************************************************/
void vtMgrZeroChnRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                      U32                      chnUnique,
                      VTMNG_CHN_STATS         *chnStats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s zeroing CHN statistics (chnUnique=0x%08X)\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ),
                chnUnique );
    }
    else
    {
        printf( "CHN statistics zeroed successfully (chnUnique=0x%08X):\n", chnUnique );
    }
}

/*****************************************************************************
 * vtMgrZeroTotalRsp - handle rx of response to ZERO TOTAL request
 *****************************************************************************/
void vtMgrZeroTotalRsp( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                        VTMNG_ST_ENTRY        *stats )
{
    if (result != VS_SUCCESS)
    {
        printf( "Error %s zeroing TOTAL statistics\n",
                vtMngValueName( VTMNG_VALUE_RESULT, result ) );
    }
    else
    {
        printf( "TOTAL statistics zeroed successfully\n" );
    }
}

/*****************************************************************************
 * vtMgrVtpLevelTrap - handle rx of VTP TRAP
 *****************************************************************************/
void vtMgrVtpLevelTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                        VTMNG_NOTIF_INFO       *info,
                        VTMNG_VTPLVL_NOTIF     *vtpLvl )
{
    printf( "\n" );
    printf( "VTP LEVEL TRAP 0x%08X (from %s)\n", info->eventId, vtpAddr->hostName );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrVtpErrorTrap - handle rx of VTP ERROR TRAP
 *****************************************************************************/
void vtMgrVtpErrorTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                        VTMNG_NOTIF_INFO     *info )
{
    printf( "\n" );
    printf( "VTP ERROR TRAP 0x%08X (from %s)\n", info->eventId, vtpAddr->hostName );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrAppConnTrap - handle rx of APP CONN TRAP
 *****************************************************************************/
void vtMgrAppConnTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                       U32                    appUnique,
                       VTMNG_NOTIF_INFO      *info,
                       VTMNG_APPCON_NOTIF    *appCon )
{
    printf( "\n" );
    printf( "APP CONN TRAP 0x%08X (from %s) (appUnique=0x%08X)\n", info->eventId,  vtpAddr->hostName, appUnique );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrMonProcTrap - handle rx of MONITORED PROCESS TRAP
 *****************************************************************************/
void vtMgrMonProcTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                       U32                    monUnique,
                       VTMNG_NOTIF_INFO      *info,
                       VTMNG_MONPROC_NOTIF   *monProc )
{
    printf( "\n" );
    printf( "MON PROC TRAP 0x%08X (from %s) (monUnique=0x%08X)\n", info->eventId,  vtpAddr->hostName, monUnique );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrChnCreateTrap - handle rx of CHN CREATE TRAP
 *****************************************************************************/
void vtMgrChnCreateTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                         U32                      chnUnique,
                         VTMNG_NOTIF_INFO        *info,
                         VTMNG_CHN_CREATE_NOTIF  *chnCreate )
{
    printf( "\n" );
    printf( "CHN CREATE TRAP (from %s) (chnUnique=0x%08X)\n", vtpAddr->hostName, chnUnique );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrChnStartTrap - handle rx of CHN START TRAP
 *****************************************************************************/
void vtMgrChnStartTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                        U32                       chnUnique,
                        VTMNG_NOTIF_INFO         *info,
                        VTMNG_CHN_START_NOTIF    *chnStart )
{
    printf( "\n" );
    printf( "CHN START TRAP (from %s) (chnUnique=0x%08X)\n", vtpAddr->hostName, chnUnique );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrChnStopTrap - handle rx of CHN STOP TRAP
 *****************************************************************************/
void vtMgrChnStopTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                       U32                        chnUnique,
                       VTMNG_NOTIF_INFO          *info,
                       VTMNG_CHN_STOP_NOTIF      *chnStop )
{
    printf( "\n" );
    printf( "CHN STOP TRAP (from %s) (chnUnique=0x%08X)\n", vtpAddr->hostName, chnUnique );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrChnDeadTrap - handle rx of CHN DEAD TRAP
 *****************************************************************************/
void vtMgrChnDeadTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                       U32                        chnUnique,
                       VTMNG_NOTIF_INFO          *info,
                       VTMNG_CHN_DEAD_NOTIF      *chnDead )
{
    printf( "\n" );
    printf( "CHN DEAD TRAP (from %s) (chnUnique=0x%08X)\n", vtpAddr->hostName, chnUnique );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrChnErrorTrap - handle rx of CHN ERROR TRAP
 *****************************************************************************/
void vtMgrChnErrorTrap( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                        U32                       chnUnique,
                        VTMNG_NOTIF_INFO         *info,
                        VTMNG_CHN_ERROR_NOTIF    *chnError )
{
    printf( "\n" );
    printf( "CHN ERROR TRAP (from %s) (chnUnique=0x%08X) [errCode=0x%08X]\n", vtpAddr->hostName, chnUnique, info->errCode );
    printf( "\n" );
}

/*****************************************************************************
 * vtMgrCmdNotif - handle rx of operator command (keyboard input)
 *****************************************************************************/
U8 vtMgrCmdNotif( void *userkey, S8 *command )
{
    U8      remainInPollLoop = TRUE;
    U32     result = VS_SUCCESS;

    myInfo.dest.sendkey = 0;        /* clear default send-specific key */

    if ( (strcmp( command, "q" ) == 0) ||
         (strcmp( command, "quit" ) == 0) )
    {
        remainInPollLoop = FALSE; /* do not remain in poll loop */
    }
    else if ( (command[0] == 'u') && (command[1] == ' ') )
    {   /* unique ID value specified [used by certain other commands] */
        if ( (command[2] == '0') &&
             ( (command[3] == 'x') || (command[3] == 'X') ) &&
             (strlen(command) > 4) )
        {
            myInfo.uniqueId = strtoul( &command[4], NULL, 16 );
        }
        else
        {
            myInfo.uniqueId = strtoul( &command[2], NULL, 10 );
        }
    }
    else if (strcmp( command, "apps" ) == 0)
    {
        result = vtMngGetAppList( &myInfo.dest );
    }
    else if (strcmp( command, "mons" ) == 0)
    {
        result = vtMngGetMonList( &myInfo.dest );
    }
    else if (strcmp( command, "chans" ) == 0)
    {
        result = vtMngGetChnList( &myInfo.dest );
    }
    else if (strcmp( command, "vtp" ) == 0)
    {
        result = vtMngGetVtp( &myInfo.dest );
    }
    else if (strcmp( command, "app" ) == 0)
    {
        result = vtMngGetApp( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "mon" ) == 0)
    {
        result = vtMngGetMon( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "chan" ) == 0)
    {
        result = vtMngGetChn( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "total" ) == 0)
    {
        result = vtMngGetStTotal( &myInfo.dest );
    }
    else if (strcmp( command, "currmin" ) == 0)
    {
        result = vtMngGetStCurrMin( &myInfo.dest );
    }
    else if (strcmp( command, "min" ) == 0)
    {
        result = vtMngGetHistPerMin( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "hhr" ) == 0)
    {
        result = vtMngGetHistPerHHr( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "setvtp" ) == 0)
    {
        result = vtMngSetVtp( &myInfo.dest, &myInfo.vtpCfg, &myInfo.vtpMask );
    }
    else if (strcmp( command, "restartvtp" ) == 0)
    {
        myInfo.dest.sendkey = VTMNG_VTP_E_RESTART;      /* set send-specific key to type of event (for display on response) */
        result = vtMngEventVtp( &myInfo.dest, VTMNG_VTP_E_RESTART );
    }
    else if (strcmp( command, "rebootvtp" ) == 0)
    {
        myInfo.dest.sendkey = VTMNG_VTP_E_REBOOT;       /* set send-specific key to type of event (for display on response) */
        result = vtMngEventVtp( &myInfo.dest, VTMNG_VTP_E_REBOOT );
    }
    else if (strcmp( command, "setapp" ) == 0)
    {
        result = vtMngSetApp( &myInfo.dest, myInfo.uniqueId, &myInfo.appCfg, &myInfo.appMask );
    }
    else if (strcmp( command, "eventapp" ) == 0)
    {
        printf( "ERROR: no application-level events currently defined.\n" );
    }
    else if (strcmp( command, "setmon" ) == 0)
    {
        result = vtMngSetMon( &myInfo.dest, myInfo.uniqueId, &myInfo.monCfg, &myInfo.monMask );
    }
    else if (strcmp( command, "termmon" ) == 0)
    {
        myInfo.dest.sendkey = VTMNG_MON_E_TERMINATE;    /* set send-specific key to type of event (for display on response) */
        result = vtMngEventMon( &myInfo.dest, myInfo.uniqueId, VTMNG_MON_E_TERMINATE );
    }
    else if (strcmp( command, "stopmon" ) == 0)
    {
        myInfo.dest.sendkey = VTMNG_MON_E_STOP;         /* set send-specific key to type of event (for display on response) */
        result = vtMngEventMon( &myInfo.dest, myInfo.uniqueId, VTMNG_MON_E_STOP );
    }
    else if (strcmp( command, "startmon" ) == 0)
    {
        myInfo.dest.sendkey = VTMNG_MON_E_START;        /* set send-specific key to type of event (for display on response) */
        result = vtMngEventMon( &myInfo.dest, myInfo.uniqueId, VTMNG_MON_E_START );
    }
    else if (strcmp( command, "setchan" ) == 0)
    {
        result = vtMngSetChn( &myInfo.dest, myInfo.uniqueId, &myInfo.chnCfg, &myInfo.chnMask );
    }
    else if (strcmp( command, "abortchan" ) == 0)
    {
        myInfo.dest.sendkey = VTMNG_CHN_E_ABORT;        /* set send-specific key to type of event (for display on response) */
        result = vtMngEventChn( &myInfo.dest, myInfo.uniqueId, VTMNG_CHN_E_ABORT );
    }
    else if (strcmp( command, "zerovtp" ) == 0)
    {
        result = vtMngZeroVtp( &myInfo.dest );
    }
    else if (strcmp( command, "zeroapp" ) == 0)
    {
        result = vtMngZeroApp( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "zeromon" ) == 0)
    {
        result = vtMngZeroMon( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "zerochan" ) == 0)
    {
        result = vtMngZeroChn( &myInfo.dest, myInfo.uniqueId );
    }
    else if (strcmp( command, "zerototal" ) == 0)
    {
        result = vtMngZeroTotal( &myInfo.dest );
    }
    else if (strcmp( command, "?" ) == 0)
    {
        printf( "COMMANDS:\n" );
        printf( "  quit | q     - quit VTMGR\n" );
        printf( "  u <uniqueID> - set unique ID (used on certain other commands [UID])\n" );
        printf( "  apps         - get list of all applications connected to the VTP\n" );
        printf( "  mons         - get list of all monitored processes on the VTP\n" );
        printf( "  chans        - get list of all channels defined on the VTP\n" );
        printf( "  vtp          - show VTP-level information\n" );
        printf( "  app          - show application-level information (appUinque = UID)\n" );
        printf( "  mon          - show monitored process-specific information (monUinque = UID)\n" );
        printf( "  chan         - show channel-level information (chnUinque = UID)\n" );
        printf( "  total        - show total statistics\n" );
        printf( "  currmin      - show statistics for current 1-minute interval\n" );
        printf( "  min          - show per-minute historgram entry (minutesAgo = UID)\n" );
        printf( "  hhr          - show per-half-hour historgram entry (halfHoursAgo = UID)\n" );
        printf( "  setvtp       - send VTP-level configuration [BLANK config and mask in use]\n" );
        printf( "  restartvtp   - cause the VTP to restart all transcoder processes\n" );
        printf( "  rebootvtp    - cause the VTP to reboot\n" );
        printf( "  setapp       - set application-level config. [BLANK config and mask in use]\n" );
        printf( "  eventapp     - issue an event to the controlling application connection\n" );
        printf( "  setmon       - set monitored process config. [BLANK config and mask in use]\n" );
        printf( "  addmon       - issue an event to add a new process to the set being monitored\n" );
        printf( "  remmon       - issue an event to remove a process from the set being monitored\n" );
        printf( "  movemon      - issue an event to move a process to a different order in startup sequence\n" );
        printf( "  termmon      - issue an event to terminate the process being monitored (triggering recovery)\n" );
        printf( "  stopmon      - issue an event to stop the process without process being automatically restarted\n" );
        printf( "  startmon     - issue an event to start a monitored process that had been stopped\n" );
        printf( "  setchan      - set channel-level configuration [BLANK config and mask in use]\n" );
        printf( "  abortchan    - abort the channel (UID)\n" );
        printf( "  zerovtp      - zero all VTP-level statistics\n" );
        printf( "  zeroapp      - zero application-level statistics (appUnique = UID)\n" );
        printf( "  zeromon      - zero monitored process statistics (monUnique = UID)\n" );
        printf( "  zerochan     - zero channel-level statistics (chnUnique = UID)\n" );
        printf( "  zerototal    - zero total (overall) statistics\n" );
    }
    else if (strlen( command ) > 0)
    {
        printf( "ERROR: unknown command (%s)\n", command );
    }

    if (result != VS_SUCCESS)
    {
        printf( "ERROR 0x%08X occurred while issuing VT Management request.\n", result );
    }

    if (remainInPollLoop)
    {
        printf( "VTMGR> " );
    }

    return( remainInPollLoop );
}

/*###########################################################################*
 * MAIN
 *###########################################################################*/
int main(int argc, char *argv[])
{
    char           *myname;
    int             arg_index = 1;          /* index to next parameter to parse */
    U32             result = VS_SUCCESS;    /* no error encountered yet */
    char           *vtpAddress      = NULL;
    char           *eventLog        = NULL;
    char           *cmdString;
    U32             reqPort         = VTMNG_PORT_SELECT;    /* select any unused UDP port */
    U32             trapPort        = VTMNG_PORT_DISABLED;  /* do not register for traps */
    U32             dbgMask         = MY_VTMNG_DEFAULT_LOG_MASK;
    U8              regResponses    = FALSE;
    U8              regTraps        = FALSE;
    U8              regKeyboard     = FALSE;
    VTMNG_UPCALLS   upcalls;

    /* ----- process command-line arguments ----- */
    myname = argv[0];
    printf( "%s V%d.%d - Copyright 2000-2007: NMS Communications Corporation\n",
            myname, MH_VERSION, MH_REVISION );

    while (arg_index < argc)
    {
        if ( (argv[arg_index][0] != '-') || (strlen(argv[arg_index]) < 2) )
        {
            printf( "!!! ERROR: Expecting parameters in form: -<option> [<value>]\n" );
            printf( "           [use -? (or -h) to view options]\n" );
            exit(1);
        }

        switch( argv[arg_index][1] )
        {
            case 'v':       /* VTP address */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -v <vtpAddress>\n" );
                    exit(1);
                }

                vtpAddress = argv[arg_index + 1];
                arg_index += 2;
                break;

            case 'f':       /* log filename (eventLog) */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -f <eventLog>\n" );
                    exit(1);
                }

                eventLog = argv[arg_index + 1];
                arg_index += 2;

                /* by specifying an event log file it is implied that vtmgr should register to receive traps */
                if (trapPort == VTMNG_PORT_DISABLED)
                {   /* specific trap port number has not been provided so set to default (well-known) UDP port */
                    trapPort = VT_MANAGE_NOTIF_PORT;
                }
                break;

            case 'r':       /* request response port */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -r <reqPort>\n" );
                    exit(1);
                }

                cmdString = argv[arg_index + 1];
                reqPort = strtol( cmdString, NULL, 10 );
                arg_index += 2;
                break;

            case 't':       /* asynchronous event (trap) port */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -t <trapPort>\n" );
                    exit(1);
                }

                cmdString = argv[arg_index + 1];
                trapPort = strtol( cmdString, NULL, 10 );
                if (trapPort == 0)
                {   /* since the default for trapPort is VTMNG_PORT_DISABLED (0), specifying -t 0 is interpreted to mean "use well-known port" */
                    trapPort = VT_MANAGE_NOTIF_PORT;
                }
                arg_index += 2;
                break;

            case 'c':       /* specify level of information to be traced on channel notifications */
                if (arg_index + 1 >= argc)
                {
                    printf( "!!! ERROR: Expecting -c <chnTrcLvl>\n" );
                    exit(1);
                }

                cmdString = argv[arg_index + 1];
                if (strcmp( cmdString, "off" ) == 0)
                {   /* all channel indication tracing is OFF */
                    dbgMask &= ~VTMNG_LOG_CHN;
                    dbgMask &= ~VTMNG_LOG_CHD;
                }
                else if (strcmp( cmdString, "state" ) == 0)
                {   /* trace channel state change information only */
                    dbgMask |= VTMNG_LOG_CHN;
                    dbgMask &= ~VTMNG_LOG_CHD;
                }
                else if (strcmp( cmdString, "all" ) == 0)
                {   /* trace all channel-level indication information */
                    dbgMask |= VTMNG_LOG_CHN;
                    dbgMask |= VTMNG_LOG_CHD;
                }
                else
                {
                    printf( "!!! ERROR: Expecting -c <chnTrcLvl>\n" );
                    printf( "  where <chnTrcLvl> (channel trace level) is one of the following:\n" );
                    printf( "    off   = all channel indication tracing is OFF\n" );
                    printf( "    state = trace channel state change information only\n" );
                    printf( "    all   = trace all channel-level indication information\n" );
                    printf( "  DEFAULT level = state\n" );
                    exit(1);
                }
                arg_index += 2;
                break;

            case 'd':       /* debug log mask */
                cmdString = argv[arg_index + 1];
                if ( (arg_index + 1 >= argc) ||
                     (cmdString[0] == '?') )
                {
                    printf( "!!! ERROR: Expecting -d <dbgMask>\n" );
                    printf( "  where debug log bits include:\n" );
                    printf( "    [FUL] (1<<23) Trace management messages (summaries)\n" );
                    printf( "    [MNG] (1<<22) Trace management messages (full messages)\n" );
                    printf( "    [CHD] (1<<21) Trace detailed channel indication information\n" );
                    printf( "    [E2C] (1<<20) Trace event information to console\n" );
                    printf( "    [MON] (1<<19) Monitored Process lost / recovered indication\n" );
                    printf( "    [CHN] (1<<18) Channel state change indication\n" );
                    printf( "    [APP] (1<<17) Control Application connected / disconnected\n" );
                    printf( "    [VTP] (1<<16) VTP-level threshold crossed\n" );
                    printf( "    [WRN] (1<<1)  Warning condition\n" );
                    printf( "    [ERR] (1<<0)  Error detected\n" );
                    printf( "    ----|----|FMD2|MCAV|----|----|----|--WE\n" );
                    printf( "  DEFAULT mask = 0x%08X\n", MY_VTMNG_DEFAULT_LOG_MASK );
                    exit(1);
                }

                if ( (cmdString[0] == '0') &&
                     ( (cmdString[1] == 'x') || (cmdString[1] == 'X') ) &&
                     (strlen(cmdString) > 2) )
                {
                    dbgMask = strtoul( &cmdString[2], NULL, 16 );
                }
                else
                {
                    dbgMask = strtoul( cmdString, NULL, 10 );
                }
                arg_index += 2;
                break;

            case 'R':       /* register for responses */
                regResponses = TRUE;
                arg_index += 1;
                break;

            case 'T':       /* register for TRAPs */
                regTraps = TRUE;
                arg_index += 1;
                break;

            case 'K':       /* register for KEYBOARD */
                regKeyboard = TRUE;
                arg_index += 1;
                break;

            case '?':
            case 'h':
                printf( "Command line options: [-v <vtpAddr>] [-f <eventLog>]\n" );
                printf( "                      [-r <reqPort>] [-t <trapPort>] [-c <chnTrcLvl>]\n" );
                printf( "                      [-d <dbgMask>]\n" );
                printf( "                      [-R] [-T] [-K] \n" );
                printf( "  where\n" );
                printf( "    -v <vtpAddr>  = set destination VTP address\n" );
                printf( "    -f <eventLog> = set filename to log async events to [and reg for traps]\n" );
                printf( "    -r <reqPort>  = specify UDP port to issue requests & receive responses over\n" );
                printf( "                    DEFAULT = select any available port\n" );
                printf( "    -t <trapPort> = specify UDP port number to listen on for asynchronous traps\n" );
                printf( "                    DEFAULT = do not listen for traps (unless -f specified)\n" );
                printf( "    -c <chnTrcLvl>= information traced on channel events [off|state|all]\n" );
                printf( "                    DEFAULT = off (no information traced on channel events)\n" );
                printf( "    -d <dbgMask>  = specify debug log mask (use -d ? for more details)\n" );
                printf( "    -R            = register for management response handling\n" );
                printf( "    -T            = register for management TRAP handling\n" );
                printf( "    -K            = register for keyboard command handling\n" );
                exit(0);
                break;

            default:
                printf( "!!! ERROR: Unknown option [-%c]; use -? (or -h) to view help\n",
                        argv[arg_index][1] );
                exit(1);
        }
    }

    /* ----- setup my own internal information ----- */
    memset( (void *)&myInfo, 0, sizeof(MY_VTMNG_INFO) );
    myInfo.id = MY_VTMNG_ID;
    if (vtpAddress == NULL)
    {   /* no destination address was specified so setup to use the local loopback address */
        strcpy( myInfo.dest.hostName, "127.0.0.1" );
    }
    else
    {   /* destination address was specified so remember that address for future requests */
        strncpy( myInfo.dest.hostName, vtpAddress, VTMNG_LONGNAME_SZ );
    }

    /* ----- initialize the VTMNG API ----- */
    memset( (void *)&upcalls, 0, sizeof(VTMNG_UPCALLS) );       /* start with all upcalls handled internally (by VTMNG API) */

    if (regResponses)
    {   /* register to handle all received management responses */
        upcalls.vtGetAppListRsp     = vtMgrGetAppListRsp;
        upcalls.vtGetMonListRsp     = vtMgrGetMonListRsp;
        upcalls.vtGetChnListRsp     = vtMgrGetChnListRsp;
        upcalls.vtGetVtpRsp         = vtMgrGetVtpRsp;
        upcalls.vtGetAppRsp         = vtMgrGetAppRsp;
        upcalls.vtGetMonRsp         = vtMgrGetMonRsp;
        upcalls.vtGetChnRsp         = vtMgrGetChnRsp;
        upcalls.vtGetStTotalRsp     = vtMgrGetStTotalRsp;
        upcalls.vtGetStCurrMinRsp   = vtMgrGetStCurrMinRsp;
        upcalls.vtGetHistPerMinRsp  = vtMgrGetHistPerMinRsp;
        upcalls.vtGetHistPerHHrRsp  = vtMgrGetHistPerHHrRsp;
        upcalls.vtSetVtpRsp         = vtMgrSetVtpRsp;
        upcalls.vtEventVtpRsp       = vtMgrEventVtpRsp;
        upcalls.vtSetAppRsp         = vtMgrSetAppRsp;
        upcalls.vtEventAppRsp       = vtMgrEventAppRsp;
        upcalls.vtSetMonRsp         = vtMgrSetMonRsp;
        upcalls.vtEventMonRsp       = vtMgrEventMonRsp;
        upcalls.vtSetChnRsp         = vtMgrSetChnRsp;
        upcalls.vtEventChnRsp       = vtMgrEventChnRsp;
        upcalls.vtZeroVtpRsp        = vtMgrZeroVtpRsp;
        upcalls.vtZeroAppRsp        = vtMgrZeroAppRsp;
        upcalls.vtZeroMonRsp        = vtMgrZeroMonRsp;
        upcalls.vtZeroChnRsp        = vtMgrZeroChnRsp;
        upcalls.vtZeroTotalRsp      = vtMgrZeroTotalRsp;
    }

    if (regTraps)
    {   /* register to handle all received management TRAPs */
        upcalls.vtVtpLevelTrap      = vtMgrVtpLevelTrap;
        upcalls.vtVtpErrorTrap      = vtMgrVtpErrorTrap;
        upcalls.vtAppConnTrap       = vtMgrAppConnTrap;
        upcalls.vtMonProcTrap       = vtMgrMonProcTrap;
        upcalls.vtChnCreateTrap     = vtMgrChnCreateTrap;
        upcalls.vtChnStartTrap      = vtMgrChnStartTrap;
        upcalls.vtChnStopTrap       = vtMgrChnStopTrap;
        upcalls.vtChnDeadTrap       = vtMgrChnDeadTrap;
        upcalls.vtChnErrorTrap      = vtMgrChnErrorTrap;
    }

    if (regKeyboard)
    {   /* register to handle all received operator commands (via keyboard) */
        upcalls.vtCmdNotif          = vtMgrCmdNotif;
    }

    result = vtMngInit( (void *)&myInfo,    /* user-controlled key to be provided on all notifications */
                        &upcalls,           /* set of upcall functions (received responses|traps [keyboard]) */
                        reqPort,            /* UDP port number to use for issuing requests (and receiving responses) */
                        trapPort,           /* UDP port number to listen on for receiving traps */
                        dbgMask,            /* mask of active trace bits */
                        vtpAddress,         /* OPTIONAL name of initial destination */
                        eventLog );         /* OPTIONAL filename used to log async events to disk */
    if (result != VS_SUCCESS)
    {
        printf( "!!! ERROR: 0x%08X returned from vtMngInit.\n",
                result );
        exit(1);
    }

    if (regKeyboard)
    {   /* registered to handle all received operator commands (via keyboard) so put up prompt */
        printf( "VTMGR> " );
    }

    /* ----- now allow the VTMNG API to control program flow from here (upcalling on registered receive) ----- */
    result = vtMngPollLoop( TRUE, 0 );            /* stay in polling loop forever */
    if (result != VS_SUCCESS)
    {
        printf( "!!! ERROR: 0x%08X returned from vtMngPollLoop.\n",
                result );
        exit(1);
    }

    /* ----- cleanly shutdown the VTMNG API ----- */
    result = vtMngShutdown( );
    if (result != VS_SUCCESS)
    {
        printf( "!!! ERROR: 0x%08X returned from vtMngShutdown.\n",
                result );
        exit(1);
    }

    return (0);
}
